 /****************************************************************/
 /*          S A S   A U T O C A L L   L I B R A R Y             */
 /*                                                              */
 /*    NAME: MktEval                                             */
 /*   TITLE: Evaluate the Goodness of an Experimental Design     */
 /* PRODUCT: STAT                                                */
 /*  SYSTEM: ALL                                                 */
 /*    KEYS: marketing research, designs                         */
 /*   PROCS: IML, SUMMARY                                        */
 /*    DATA:                                                     */
 /* SUPPORT: saswfk                      UPDATE:  25Jul2010      */
 /*     REF: See "Marketing Research Methods in SAS" at the      */
 /*          following URL for more information on this macro.   */
 /*                                                              */
 /* http://support.sas.com/techsup/tnote/tnote_stat.html#market  */
 /*                                                              */
 /*    MISC: This macro creates a global macro variable          */
 /*             _MKTEVALSUM _MKTVER                              */
 /*                                                              */
 /*          The following SAS products are required to run      */
 /*          this macro: BASE, STAT, and IML.                    */
 /****************************************************************/

 /*---------------------------------------------------------------------

This macro prints information to help you evaluate the suitability
of an experimental design.  This macro prints canonical correlations
between factors along with 1-way, 2-way, and n-way frequencies.
Given a coded experimental design matrix X, the variances of the
parameter estimates are proportional to inv(X'X).  This matrix can
be printed with PROC OPTEX.  However, this matrix can be large; it
has one column for each parameter.  Some researchers prefer a more
compact description of the association between factors.  For m
categorical factors or CLASS variables, this macro creates an
m x m matrix of canonical correlations between the m coded factors.
For example, in this design, there is a small correlation between
X1 and X2 but all of the other factors are orthogonal.

   %mktex(2 ** 2 3 ** 3, n=18, seed=205)

   %mkteval;

Here is the output.

                  Canonical Correlations Between the Factors
            There are 0 Canonical Correlations Greater Than 0.316

                          x1      x2      x3      x4      x5

                  x1    1       0.11    0       0       0
                  x2    0.11    1       0       0       0
                  x3    0       0       1       0       0
                  x4    0       0       0       1       0
                  x5    0       0       0       0       1


                            Summary of Frequencies
            There are 0 Canonical Correlations Greater Than 0.316
                      * - Indicates Unequal Frequencies

                            Frequencies

                   x1       9 9
                   x2       9 9
                   x3       6 6 6
                   x4       6 6 6
                   x5       6 6 6
              *    x1 x2    4 5 5 4
                   x1 x3    3 3 3 3 3 3
                   x1 x4    3 3 3 3 3 3
                   x1 x5    3 3 3 3 3 3
                   x2 x3    3 3 3 3 3 3
                   x2 x4    3 3 3 3 3 3
                   x2 x5    3 3 3 3 3 3
                   x3 x4    2 2 2 2 2 2 2 2 2
                   x3 x5    2 2 2 2 2 2 2 2 2
                   x4 x5    2 2 2 2 2 2 2 2 2
                   N-Way    1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1

A canonical correlation is the maximum correlation between linear
combinations of the coded factors.  The hoped for values (ones on the
diagonal and zeros off the diagonal) are printed with zero decimal
places.  All other values are printed with a 4.2 format so that they are
easily noticeable.  The macro counts and reports the number of large
canonical correlations, by default those greater than 0.316 (r squared =
0.1 or 10%).  In this example, only one canonical correlation is greater
than zero indicating that the design is nearly orthogonal.

The frequencies show that this design is balanced (all one-way
frequencies are equal), nearly orthogonal (only one set of two-way
frequencies are unequal), and each profile appears once (all n-way
frequencies are one).

Here is another example.

   %mktex(3 ** 15, n=36, seed=17)

   %mkteval;

Here is some of the output.

                  Canonical Correlations Between the Factors
            There are 0 Canonical Correlations Greater Than 0.316

    x1   x2   x3   x4   x5   x6   x7   x8   x9   x10  x11  x12  x13  x14  x15

x1  1    0    0    0    0    0    0    0    0    0    0    0    0    0    0
x2  0    1    0    0    0    0    0    0    0    0    0    0    0    0    0
x3  0    0    1    0    0    0    0    0    0    0    0    0    0    0    0
x4  0    0    0    1    0    0    0    0    0    0    0    0    0    0    0
x5  0    0    0    0    1    0    0    0    0    0    0    0    0    0    0
x6  0    0    0    0    0    1    0    0    0    0    0    0    0    0    0
x7  0    0    0    0    0    0    1    0    0    0    0    0    0    0    0
x8  0    0    0    0    0    0    0    1    0    0    0    0    0    0    0
x9  0    0    0    0    0    0    0    0    1    0    0    0    0    0    0
x10 0    0    0    0    0    0    0    0    0    1    0    0    0    0    0
x11 0    0    0    0    0    0    0    0    0    0    1    0    0    0    0
x12 0    0    0    0    0    0    0    0    0    0    0    1    0    0    0
x13 0    0    0    0    0    0    0    0    0    0    0    0    1    0.25 0
x14 0    0    0    0    0    0    0    0    0    0    0    0    0.25 1    0
x15 0    0    0    0    0    0    0    0    0    0    0    0    0    0    1

When is a canonical correlation too big?  You will have to decide that
for yourself.  In part, the answer depends on the factors and how the
design will be used.  A high correlation between the client's and the
main competitor's price factor is a serious problem meaning you will
need to use a different design.  On the other hand, a moderate
correlation in a choice design between one brand's minor attribute and
another brand's minor attribute may be perfectly fine.

------------------------------------------------------------------------

 DISCLAIMER:

       THIS INFORMATION IS PROVIDED BY SAS INSTITUTE INC. AS A SERVICE
 TO ITS USERS.  IT IS PROVIDED "AS IS".  THERE ARE NO WARRANTIES,
 EXPRESSED OR IMPLIED, AS TO MERCHANTABILITY OR FITNESS FOR A
 PARTICULAR PURPOSE REGARDING THE ACCURACY OF THE MATERIALS OR CODE
 CONTAINED HEREIN.

----------------------------------------------------------------------*/

%*===================== Macro Argument List Start =====================;

%macro mkteval(   /*--------------------------------------------------*/
                  /*                                                  */
help,             /* nonblank, list options                           */
                  /*                                                  */
                  /* This macro helps you evaluate the suitability of */
                  /* an experimental design after using the %MktEx    */
                  /* macro to generate it.  This macro reports on     */
                  /* canonical correlations between factors and level */
                  /* frequencies.                                     */
                  /*--------------------------------------------------*/
data=,            /* Input data set with the experimental design.     */
                  /* By default, the macro uses the last data set     */
                  /* created.                                         */
                  /*--------------------------------------------------*/
vars=,            /* specifies the factors in the design.             */
                  /* By default, all numeric variables are used.      */
factors=,         /* Alias for VARS=.                                 */
                  /*--------------------------------------------------*/
outcorr=corr,     /* Output data set for the canonical correlation    */
                  /* matrix.                                          */
                  /*--------------------------------------------------*/
outcb=cb,         /* Output data set for the canonical correlation    */
                  /* matrix within each block.                        */
                  /*--------------------------------------------------*/
outlist=list,     /* Output data set for the list of largest          */
                  /* canonical correlations.                          */
                  /*--------------------------------------------------*/
outfreq=freq,     /* Output data set for the frequencies.             */
                  /*--------------------------------------------------*/
outfsum=fsum,     /* Output data set for the frequency summaries.     */
                  /*--------------------------------------------------*/
list=0.316,       /* List any correlations bigger than this number.   */
                  /* The default is the square root of 0.1.  Typical  */
                  /* usage is something like list=0.3 or list=0.5.    */
                  /*--------------------------------------------------*/
freqs=1 2 n,      /* Prints by default 1-way, 2-way, and n-way        */
                  /* frequencies.  Do not specify the exact number of */
                  /* ways instead of n.  For ways other than "n", the */
                  /* macro checks for zero cells.  For n-ways, the    */
                  /* macro does not check for zero cells.  Only the   */
                  /* full-factorial design will have nonzero cells,   */
                  /* so specifying something like FREQS=1 2 20 will   */
                  /* make the macro take a LONG time and create HUGE  */
                  /* data sets, whereas FREQS=1 2 n runs very         */
                  /* reasonably.                                      */
                  /*--------------------------------------------------*/
format=,          /* Format for printing canonical correlations.      */
                  /*--------------------------------------------------*/
blocks=,          /* Blocking variable.  Prints separate canonical    */
                  /* correlations within each block.                  */
                  /*--------------------------------------------------*/
print=short,      /* Controls the printing of the results.  Values    */
                  /* include:                                         */
                  /* corr    - canonical correlations                 */
                  /* list    - list of canonical correlations         */
                  /*           > LIST= value.                         */
                  /* ordered - list ordered by variable names.        */
                  /* nonzero - list ordered by variable names and     */
                  /*           sets LIST=1e-6 to print nonzero        */
                  /*           canonical correlations.                */
                  /* freqs   - frequencies, controlled by the FREQS=  */
                  /*           option.                                */
                  /* summ    - frequency summaries.                   */
                  /* block   - canonical correlations within blocks.  */
                  /* all     - all of the above.                      */
                  /* short   - corr list summ block (default).        */
                  /* noprint - no printed output but make the data    */
                  /*           sets.                                  */
);                /*--------------------------------------------------*/
                  /* Global Macro Variable:                           */
                  /* _mktevalsum - sum of lower triangle of           */
                  /*               canonical correlation matrix.      */
                  /*--------------------------------------------------*/
                  /* This macro specifies OPTIONS NONOTES throughout  */
                  /* most of its execution.  If you want to see all   */
                  /* the notes, submit the statement                  */
                  /*    %let mktopts = notes;                         */
                  /* before running the macro.                        */
                  /*--------------------------------------------------*/

;*=========================== Macro Start =============================;

*------initial macro stuff------;
%global _mktevalsum _mktver;
%local  abort anystar bigcorr blockvar defdata dupprof k miss missb
        savedsnf nway pcorr pcorrb pfreq plist psumm savenote setnote
        deffact time titles tnum t1 t2 saveopts maxlen pord justdata
        islist holdlast timeopt localver;

%let time     = %sysfunc(datetime());
%let timeopt  = %scan(STIMER STATS, 1 + (&sysscp = OS));
%let savenote = %sysfunc(getoption(notes));
%let saveopts = &savenote missing="%sysfunc(getoption(missing))"
                %sysfunc(getoption(generic)) %sysfunc(getoption(&timeopt))
                %sysfunc(getoption(dsnferr)) %sysfunc(getoption(fmterr))
                %sysfunc(getoption(serror));
%let setnote  = nonotes;
%let maxlen   = 32700;
%let localver = 25Jul2010;
%let _mktver  = &localver;
%let holdlast = &syslast;
%let abort    = 0;

options noserror;
%if %nrquote(&&mktopts) ne %nrstr(&)mktopts %then %do;
   %let k = %upcase(&mktopts);
   %if %index(&k, CHECKVERSION) %then %do; options serror; %goto mendit; %end;
   %if %index(&k, NOTES)   %then %let setnote = notes;
   %if %index(&k, VERSION) %then %put MktEval macro version &localver.;
   %end;
options serror;

options &setnote generic missing='.';

%if %nrbquote(&help) ne %then %do; %mkthelp(mkteval) %goto endit; %end;

data _null_;
   ok = 1;
   length __l $ 67;
   call symput('dupprof', '0');            /* duplicate profiles?    */
   call symput('nway',    '0');            /* n-way frequencies?     */
   call symput('tnum',    '1');            /* title number           */
   call symput('anystar', '0');            /* print "*" title?       */
   call symput('bigcorr', '.');
   call symput('miss',    '0');
   call symput('islist',  '0');
   __l = symget('data');
   def = '0';
   if __l = ' ' or                         /* default, last data set */
      upcase(__l) = '_LAST_' then do;
      call symput('data', trim(symget('syslast')));
      def = '1';
      end;
   call symput('defdata', def);

   *------check FACTORS=------;
   if symget('factors') ne ' ' and symget('vars') ne ' ' then do;
      put 'ERROR: You must specify FACTORS= or VARS=, but not both.';
      ok = 0;
      end;
   if symget('factors') eq ' ' then call symput('factors', symget('vars'));
   if symget('factors') eq ' ' then call symput('factors', '_numeric_');
   call symput('deffact', put(lowcase(symget('factors')) eq '_numeric_', 1.));

   __l = symget('blocks');
   if __l = ' ' then call symput('blocks', '0');
   else do;
      call symput('blocks', '1');
      call symput('blockvar', trim(__l));
      end;

   if (sysprod('iml') + sysprod('stat')) ne 2 then do;
      put 'ERROR: SAS/STAT and SAS/IML must be licensed '
          'to run this macro.';
      ok = 0;
      end;

   if _error_ or not ok then call symput('abort', '1');
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------mention data sets, if they are invalid, quit------;
filename __dummy dummy;
proc printto log=__dummy; run;
%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

%let savedsnf = %sysfunc(getoption(dsnferr)) %sysfunc(getoption(fmterr));
options nodsnferr nofmterr;
%if %nrbquote(&data) ne %then %do;
   data __tmpedata; set &data; run;
   %if &syserr > 4 %then %let abort = 1;
   %end;
%if %nrbquote(&outcorr) ne %then %do;
   data _null_; set &outcorr; stop; run;
   %if &syserr > 4 %then %let abort = 1;
   %end;
%if %nrbquote(&outcb) ne %then %do;
   data _null_; set &outcb; stop; run;
   %if &syserr > 4 %then %let abort = 1;
   %end;
%if %nrbquote(&outlist) ne %then %do;
   data _null_; set &outlist; stop; run;
   %if &syserr > 4 %then %let abort = 1;
   %end;
%if %nrbquote(&outfreq) ne %then %do;
   data _null_; set &outfreq; stop; run;
   %if &syserr > 4 %then %let abort = 1;
   %end;
%if %nrbquote(&outfsum) ne %then %do;
   data _null_; set &outfsum; stop; run;
   %if &syserr > 4 %then %let abort = 1;
   %end;
options &savedsnf;

proc printto log=log; run;
%if &syserr > 4 %then %let abort = 2;

%if &abort eq 1 %then %do;
   %put ERROR: One or more data set names are invalid:;
   %put %str(       )data=&data;
   %put %str(       )outcorr=&outcorr;
   %put %str(       )outcb=&outcb;
   %put %str(       )outlist=&outlist;
   %put %str(       )outfreq=&outfreq;
   %put %str(       )outsum=&outfsum;
   %end;

%if &abort %then %goto endit;

%let data = __tmpedata;

*------count titles------;
proc sql;
   create view __tmpeview as select * from dictionary.titles;
   quit;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

data _null_;
   set __tmpeview end=eof;
   if type = 'T' then ntitles + 1;
   if eof then call symput('tnum', compress(put(ntitles + 1, 2.)));
   if _error_ then call symput('abort', '1');
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------check for empty (default var list) input data set------;
%if &deffact %then %do;
   %let deffact = 0;
   proc contents data=&data noprint out=__tmpeval(keep=type); run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

   data _null_;
      set __tmpeval end=eof;
      n + (type eq 1);
      if eof then call symput('deffact', put(n, 12. -L));
      if _error_ then call symput('abort', '1');
      run;

   %if &syserr > 4 %then %let abort = 1;

   %if &deffact eq 0 %then %do;
      %put ERROR: Empty design.;
      %let abort = 1;
      %end;

   %if &abort %then %goto endit;
   %end;

*------get variable list------;
proc contents data=&data(keep=&factors &blockvar) noprint
              out=__tmpeval(keep=varnum name);
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

proc sort; by varnum; run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------expand list------;
data _null_;
   length l $ &maxlen blockvar $ 32;
   retain l blockvar;
   set __tmpeval end=eof;
   if _n_ = 1 then blockvar = upcase(symget('blockvar'));
   if upcase(name) ne blockvar then l = trim(l) || ' ' || name;
   if eof then call symput('factors', trim(left(l)));
   if _error_ then call symput('abort', '1');
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------get rid of character variables------;
data __tmpeval;
   set &data(keep=&blockvar &factors);
   __j = _n_;
   if _error_ then call symput('abort', '1');
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

proc transreg data=__tmpeval maxiter=0 method=uni noz;
   model opscore(&blockvar &factors) = ide(__j);
   output replace out=__tmpeval(drop=intercept __j _name_ _type_);
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------set up parameters------;
data _null_;
   if 0 then set __tmpeval nobs=__nobs;

   if __nobs eq 0 then do;
      put "ERROR: The DATA=%nrbquote(&data) input data set is empty.";
      call symput('abort', '1');
      end;

   length __n $ 100 __l $ &maxlen;

   *------set up FREQ= list------;
   __l = symget('freqs');
   __j = index(lowcase(__l), 'n');
   if __j then do;
      substr(__l, __j, 1) = ' ';
      if __l = ' ' then __l = '0';
      call symput('freqs', trim(__l));
      call symput('nway', '1');
      end;

   *------set flags controlling what to print------;
   __l = lowcase(symget('print'));
   do __i = 1 to 10 until(__l eq ' ');
      __j = index(__l, 'cor'    ); if __j then substr(__l, __j, 4) = ' ';
      __j = index(__l, 'list'   ); if __j then substr(__l, __j, 4) = ' ';
      __j = index(__l, 'freq'   ); if __j then substr(__l, __j, 5) = ' ';
      __j = index(__l, 'sum'    ); if __j then substr(__l, __j, 4) = ' ';
      __j = index(__l, 'block'  ); if __j then substr(__l, __j, 5) = ' ';
      __j = index(__l, 'all'    ); if __j then substr(__l, __j, 3) = ' ';
      __j = index(__l, 'short'  ); if __j then substr(__l, __j, 5) = ' ';
      __j = index(__l, 'noprint'); if __j then substr(__l, __j, 7) = ' ';
      __j = index(__l, 'ordered'); if __j then substr(__l, __j, 7) = ' ';
      __j = index(__l, 'nonzero'); if __j then substr(__l, __j, 7) = ' ';
      end;
   if __l ne ' ' then do;
      put "ERROR: PRINT=%nrbquote(&print) is invalid.";
      call symput('abort', '1');
      end;

   __l = lowcase(symget('print'));
   if index(__l, 'all') then __l = 'corr list freq sum block';
   if index(__l, 'short') or __l = ' '
      then __l = trim(__l) || ' corr list sum block';

   call symput('justdata', put(index(__l, 'noprint') > 0, 1.));
   call symput('pcorr',    put(index(__l, 'cor'    ) > 0, 1.));
   call symput('pcorrb',   put(index(__l, 'block'  ) > 0, 1.));
   call symput('pfreq',    put(index(__l, 'freq'   ) > 0, 1.));
   call symput('psumm',    put(index(__l, 'sum'    ) > 0, 1.));
   call symput('pord' ,    put(index(__l, 'order'  ) > 0, 1.));
   call symput('plist',    put(index(__l, 'list'   ) > 0 and &list < 1, 1.));
   if index(__l, 'nonz') then do;
      call symput('pord', '1');
      call symput('list', '1e-6');
      end;

   *------output data set names------;
   if symget('outcorr') = ' ' then call symput('outcorr' , 'corr');
   if symget('outcb'  ) = ' ' then call symput('outcb'   , 'cb'  );
   if symget('outfreq') = ' ' then call symput('outfreq' , 'freq');
   if symget('outlist') = ' ' then call symput('outlist' , 'list');
   if symget('outfsum') = ' ' then call symput('outfsum' , 'fsum');

   __n = symget('data');
   if &defdata and
      (index(__n, compress(upcase(symget('outcorr')))) or
       index(__n, compress(upcase(symget('outcb'  )))) or
       index(__n, compress(upcase(symget('outfreq')))) or
       index(__n, compress(upcase(symget('outlist')))) or
       index(__n, compress(upcase(symget('outfsum'))))) then do;
      put 'ERROR: The DATA= data set must be different from'
                  ' the output data sets.' /
          '       You probably need to specify DATA=.';
      call symput('abort', '1');
      end;

   if _error_ then call symput('abort', '1');
   stop;
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------use IML to compute canonical correlations------;
%if &pcorr or &plist or &pord or &pcorrb or &justdata %then %do;
   proc iml;

      *------compute canonical correlations------;
      start cancorr(d) global(m, r, miss);
         if nrow(d) then do;
            r = i(m);
            ones = j(nrow(d), 1, 1);
            do i = 1 to m;
               x = designf(d[,i]);
               if ncol(x) = 0 then do j = i to m;
                  r[i,j] = .C; r[j,i] = .C; miss = 1;
                  end;
               else do;
                  x = x - ones * x[:,];
                  t = x * inv(half(x` * x));
                  do j = i + 1 to m;
                     y = designf(d[,j]);
                     if ncol(y) = 0 then do;
                        r[i,j] = .C; r[j,i] = .C; r[j,j] = .C; miss = 1;
                        end;
                     else do;
                        y = y - ones * y[:,];
                        c = y` * t;
                        a = sqrt(max(0, eigval(c` * inv(y` * y) * c)[1,1]));
                        a = a # (a > 1e-12); /* zero small r's */
                        r[j,i] = a; r[i,j] = a;
                        end;
                     end;
                  end;
               end;
            end;
         else r = j(m, m, .);
         finish;

      abort = 0; miss = 0;

      use __tmpeval;
      read all into d [colname=_name_];

      if any(nmiss(d)) then do;
         file log;
         put 'ERROR: Missing values in the design matrix are invalid.';
         abort = 1;
         end;

      if abort = 0 then do;

         *------compute canonical correlations------;
         m = ncol(d);   r = i(m);
         run cancorr(d);
         call symput('_mktevalsum', compress(char((sum(r) - m) / 2)));
         call symput('miss', compress(char(miss))); miss = 0;

         *------output canonical correlation matrix------;
         create &outcorr from r[colname=_name_ rowname=_name_];
         append from r[rowname=_name_];

         *------list of big canonical correlations------;
         k = 0;
         do i = 1 to m;
            do j = i + 1 to m;
               if r[i,j] > &list then do;
                  by2 = by2 // i;    by3 = by3 // j;
                  r_square = r_square // r[i, j];
                  k = k + 1;
                  end;
               end;
            end;

         call symput('bigcorr', compress(char(k)));

         if k then do;
            r = r_square;  r_square = r # r;
            by1 = num(char(r,10, 8));
            create &outlist var { r r_square by1 by2 by3 };
            append var { r r_square by1 by2 by3 };
            call symput('islist', '1');
            end;
         else do;
            call symput('plist' , '0');
            call symput('pord'  , '0');
            call symput('islist', '0');
            end;
         end;

      %if &blocks %then %do;
         if abort = 0 then do;
            use __tmpeval(keep=&factors);
            read all into d [colname=_name_];
            use __tmpeval(keep=&blockvar);
            read all into b [colname=bv];
            bv = bv || _name_;
            m = ncol(d);   r = i(m);
            out = (j(m, 1, 1) || r);
            create &outcb from out[colname=bv rowname=_name_];

            do block = min(b) to max(b);
               run cancorr(d[loc(b = block),]);
               out = j(m, 1, block) || r;
               append from out[rowname=_name_];
               end;
            end;

         %end;

      call symput('missb', compress(char(miss)));
      if abort then call symput('abort', '1');
      quit;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

%if &pfreq or &psumm or &justdata %then %do;
   *------compute frequencies------;
   proc summary noprint completetypes data=__tmpeval;
      class &blockvar &factors;
      output out=&outfreq;
      ways &freqs;
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

%if &pfreq or &nway or &psumm or &justdata %then %do;
   proc summary noprint nway data=__tmpeval;
      class &blockvar &factors;
      output out=&outfsum;
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

*------create descriptive label for each factor, factor pair------;
%if &pfreq or &psumm or &justdata %then %do;
   data &outfreq;
      retain __dup 0;
      length __type $ 200 __n $ 32;
      __nway = 0;
      set &outfreq %if &nway %then &outfsum(in=__nway);;
      if _type_;

      __k = n(of &blockvar &factors) + __nway;
      array __v[*] &blockvar &factors;
      if not __nway then do __j = 1 to dim(__v);
         if n(__v[__j]) then do;
            call vname(__v[__j], __n);
            __type = trim(__type) || ' ' || __n;
            end;
         __type = left(__type);
         end;
      else do;
         __type = 'N-Way';
         if __dup = 0 and _freq_ gt 1 then do;
            __dup = 1;
            call symput('dupprof', '1');
            end;
         end;
      if _error_ then call symput('abort', '1');
      drop __j __dup __n:;
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

   *------sort: 1 ways, then 2 ways, etc.------;
   proc sort out=&outfreq(drop=__k _type_);
      by __k descending _type_ &blockvar &factors;
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

   *------create summary of frequencies------;
   proc transpose data=&outfreq out=&outfsum;
      by notsorted __type;
      var _freq_;
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

*------process summary of frequencies------;
%if &psumm or &justdata %then %do;
   data &outfsum(keep=__type __star __freqs);
      set &outfsum end=__eof;
      length __freqs $ 45;
      array col[*] col:;
      if max(of col:) ne min(of col:) then do;
         __star = '*';
         __anystar + 1;
         end;
      do i = 1 to dim(col);
         if n(col[i]) then do;
            if length(__freqs) > 38 then do;
               __freqs = left(__freqs);
               output;
               __type = ' '; __freqs = ' '; __star = ' ';
               end;
            __freqs = trim(__freqs) || ' ' || compress(put(col[i], best12.));
            end;
         end;
      __freqs = left(__freqs);
      output;
      if __eof and __anystar then call symput('anystar', '1');
      if _error_ then call symput('abort', '1');
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

*------set warning, informational titles------;
%let k = %eval(&tnum+1);
%let titles = ;
%let t1 = 'WARNING: There are Duplicate Profiles in This Design';

%if &dupprof %then %do;
   %let titles = title&k &t1%str(;);
   %let k = %eval(&k+1);
   %end;

%let t1 = "There is 1 Canonical Correlation Greater Than &list";
%let t2 = "There are &bigcorr Canonical Correlations Greater Than &list";
%if &list < 1 and &bigcorr ne . %then %do;
   %if &bigcorr = 1 %then %let titles = &titles title&k &t1%str(;);
   %else %let titles = &titles title&k &t2%str(;);
   %let k = %eval(&k+1);
   %end;

%if &miss %then %let miss = title&k 'C - Constant Factor'%str(;);
%else           %let miss = ;

%if &pcorr or (&blocks and &pcorrb) or &justdata %then %do;
   proc format; value mktfmt .I = '1' 0 = '0' .C = 'C'; run;
   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

*------canonical correlation matrix------;
%if &pcorr or &justdata %then %do;

   data __tmpeval;
      set &outcorr;
      %if %nrbquote(&format) eq %then %do;
         array __x[*] &blockvar &factors;
         if n(__x[_n_]) and abs(__x[_n_] - 1) < 1e-6 then __x[_n_] = .I;
         %end;
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

%if &pcorr %then %do;
   proc print data=__tmpeval uniform label;
      title&tnum 'Canonical Correlations Between the Factors';
      &titles &miss
      format _numeric_
         %if %nrbquote(&format) eq %then mktfmt4.2; %else &format;;
      label _name_ = '00'x;
      id _name_;
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

*------print canonical correlation list------;
%if &islist and (&plist or &pord or &justdata) %then %do;

   data &outlist;
      length v1 v2 $ 32 __l $ &maxlen; retain __l; drop __l;
      set &outlist;
      if _n_ = 1 then __l = symget('blockvar') || ' ' || symget('factors');
      v1 = scan(__l, by2, ' ');
      v2 = scan(__l, by3, ' ');
      if _error_ then call symput('abort', '1');
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

   %if &plist or &justdata %then %do;
      proc sort data=&outlist out=&outlist(drop=by1-by3);
         by descending by1-by3;
         run;

      %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
      %end;

   %if &plist or &pord %then %do;
      proc print data=&outlist noobs uniform label split='#';
         title&tnum "Canonical Correlations > %nrbquote(&list) "
                    "Between the Factors";
         &titles
         format r r_square 4.2;
         label v1 = '00'x v2 = '00'x r = 'r' r_square = '#r Square';
         var v1 v2 r r_square;
         run;

      %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
      %end;

   %end;

*------print frequencies list------;
%if &pfreq %then %do;
   proc print data=&outfreq uniform label;
      title&tnum "Frequencies";
      &titles
      id __type _freq_;
      by notsorted __type;
      label _freq_ = 'Frequency' __type = 'Effects';
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

*------print frequency summary------;
%if &psumm %then %do;
   proc print data=&outfsum uniform label;
      id __star __type;
      title&tnum "Summary of Frequencies";
      &titles
      %if &anystar %then %str(title&k "* - Indicates Unequal Frequencies";);
      var __freqs;
      label __type = '00'x __freqs = 'Frequencies' __star = '00'x;
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

*------print canonical correlations within blocks------;
%if &blocks and &pcorrb %then %do;

   data __tmpeval;
      set &outcb;
      %if %nrbquote(&format) eq %then %do;
         array __x[*] &blockvar &factors;
         __i = mod(_n_ - 1, dim(__x) - 1) + 2;
         if n(__x[__i]) and abs(__x[__i] - 1) < 1e-6 then __x[__i] = .I;
         drop __i;
         %end;
      run;

    %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

   proc print data=__tmpeval uniform label;
      title&tnum 'Canonical Correlations Between the Factors by Block';
      %if &missb %then %do;
         title%eval(&tnum+1) 'C - Constant Within Block';
         %end;
      format _numeric_
             %if %nrbquote(&format) eq %then mktfmt4.2; %else &format;
             &blockvar best5.;
      label _name_ = '00'x;
      id &blockvar _name_;
      by &blockvar;
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;
   %end;

%endit:

proc datasets nolist;
   delete __tmpedata __tmpeval __tmpeview(memtype=view);
   run; quit;

%if &syserr > 4 %then %let abort = 1;

options &saveopts;
quit;

title&tnum;

%let syslast = &holdlast;

%if &abort %then %do;
   %if &syserr = 1016 or &syserr = 116  %then %put ERROR: Insufficient memory.;
   %if &syserr = 2000 or &syserr = 3000 %then
      %put ERROR: Syntax error.  Check your macro arguments for validity.;
   %put ERROR: The MKTEVAL macro ended abnormally.;
   %end;

%if %sysfunc(getoption(&timeopt)) eq &timeopt and
   %sysfunc(getoption(stimer)) eq STIMER %then %do;
   %let time = %sysfunc(round(%sysevalf(%sysfunc(datetime()) - &time), 0.01));
   %put NOTE: The MKTEVAL macro used &time seconds.;
   %end;

%mendit: %mend;
