 /****************************************************************/
 /*          S A S   A U T O C A L L   L I B R A R Y             */
 /*                                                              */
 /*    NAME: MktHelp                                             */
 /*   TITLE: List Option Names for the Mkt Family of Macros      */
 /* PRODUCT: STAT                                                */
 /*  SYSTEM: ALL                                                 */
 /*    KEYS: marketing research, designs                         */
 /*   PROCS:                                                     */
 /*    DATA:                                                     */
 /* SUPPORT: saswfk                      UPDATE:  05May2009      */
 /*     REF: See "Marketing Research Methods in SAS" at the      */
 /*          following URL for more information on this macro.   */
 /*                                                              */
 /* http://support.sas.com/techsup/tnote/tnote_stat.html#market  */
 /*                                                              */
 /*    MISC:                                                     */
 /****************************************************************/

                       /*---------------------------------------------*/
%macro mkthelp(macro); /* Provide the name of the macro (in lower     */
                       /* case) for which you want help.              */
                       /*---------------------------------------------*/

;%macro _(a); opt = &a; link doit; %mend;

data _null_;
   length opt $ 120 macro $ 8;
   retain macro ' ' code 0;
   put /// 'Option' @31 'Description' /;
  %_('\documentclass[twoside,11pt]{book}
')%_('\newcommand{\wfktitle}{Marketing Research Methods in SAS}
')%_('\newcommand{\wfktsnum}{MR-2010}
')%_('\newcommand{\wfkrpt}{\wfktsnum}
')%_('\newcommand{\wfkyear}{2010}
')%_('\newcommand{\wfkdate}{January 1}
')%_('\renewcommand{\topfraction}{1.0}
')%_('\renewcommand{\textfraction}{0.0}
')%_('\renewcommand{\bottomfraction}{1.0}
')%_('\input{ /u/saswfk/text/book2010/mymarkups.tex }
')%_('\begin{document}
')%_('\markboth{MR-2010 --- Marketing Research and Experimental Design Macro Options}%
')%_('{MR-2010 --- Marketing Research and Experimental Design Macro Options}
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf ChoicEff Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb bestcov=}{\it SAS-data-set}   & covariance matrix for the best design\\
')%_('{\ttb bestout=}{\it SAS-data-set}   & best design\\
')%_('{\ttb beta=}{\it list}              & true parameters\\
')%_('{\ttb chunks=}$n$                   & number of observations to code at once\\
')%_('{\ttb converge=}$n$                 & convergence criterion\\
')%_('{\ttb cov=}{\it SAS-data-set}       & all of the covariance matrices\\
')%_('{\ttb data=}{\it SAS-data-set}      & input choice candidate set\\
')%_('{\ttb drop=}{\it variable-list}     & variables to drop from the model\\
')%_('{\ttb fixed=}{\it variable-list}    & variable that flags fixed alternatives\\
')%_('{\ttb flags=}{\it variable-list}$|${\it n}& variables that flag the alternatives\\
')%_('{\ttb init=}{\it SAS-data-set}      & input initial design data set\\
')%_('{\ttb initvars=}{\it variable-list} & initial variables\\
')%_('{\ttb intiter=}$n$                  & maximum number of internal iterations\\
')%_('{\ttb iter=}$n$                     & maximum iterations (designs to create)\\
')%_('{\ttb maxiter=}$n$                  & maximum iterations (designs to create)\\
')%_('{\ttb model=}{\it model-specification}&{\ttb model} statement list of effects\\
')%_('{\ttb morevars=}{\it variable-list} & more variables to add to the model\\
')%_('{\ttb n=}$n$                        & number of observations\\
')%_('{\ttb nalts=}$n$                    & number of alternatives\\
')%_('{\ttb nsets=}$n$                    & number of choice sets desired\\
')%_('{\ttb options=coded}                & displays the coded candidate set \\
')%_('{\ttb options=detail}               & displays the details of the swaps \\
')%_('{\ttb options=nobeststar}           & no asterisk when a better design is found \\
')%_('{\ttb options=nocode}               & skips the PROC TRANSREG coding stage \\
')%_('{\ttb options=nodups}               & prevents duplicate choice set creation \\
')%_('{\ttb options=notests}              & suppress the hypothesis tests \\
')%_('{\ttb options=orthcan}              & orthogonalizes the candidate set \\
')%_('{\ttb options=outputall}            & outputs all designs to OUT= and COV= \\
')%_('{\ttb options=relative}             & displays final relative {\it D}-efficiency \\
')%_('{\ttb options=resrep}               & same as {\ttb options=detail} \\
')%_('{\ttb out=}{\it SAS-data-set}       & all designs data set\\
')%_('{\ttb restrictions=}{\it macro-name}& restrictions macro\\
')%_('{\ttb resvars=}{\it variable-list}  & variables for restrictions\\
')%_('{\ttb rscale=}$r$                   & relative efficiency scaling factor\\
')%_('{\ttb rscale=generic}               & equivalent to {\ttb rscale=n} ($n$ sets)\\
')%_('{\ttb rscale=alt}                   & for simple alternative-specific designs \\
')%_('{\ttb rscale=partial=}$p$ {\ttb of} $q$& $p$ of $q$ alternatives or attributes vary\\
')%_('{\ttb seed=}$n$                     & random number seed\\
')%_('{\ttb submat=}{\it number-list}     & submatrix for efficiency calculations\\
')%_('{\ttb types=}{\it integer-list}     & number of sets of each type\\
')%_('{\ttb typevar=}{\it variable}       & choice set types variable\\
')%_('{\ttb weight=}{\it weight-variable} & optional weight variable\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktex(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktex(3 ** 3, n = 3**3) /* candidate set of alternatives */
')%_('
')%_('%choiceff(data=design, model=class(x1-x3 / sta), nsets=9, options=relative,
')%_('          maxiter=10, seed=104, flags=3, beta=zero)
')%_('\end{Verbatim}
')%_('
')%_('\codebreak
')%_('%mktex(3 ** 9, n=2187) /* candidate set of choice sets */
')%_('
')%_('%mktroll(design=randomized, key=3 3, out=frac)
')%_('
')%_('%choiceff(data=frac, model=class(x1-x3 / sta), seed=104, maxiter=10,
')%_('          options=relative, nsets=9, nalts=3, beta=zero)
')%_('
')%_('
')%_('%mktex(3 ** 4, n = 9) /* design evaluation */
')%_('
')%_('%mktlab(data=design, vars=Set x1-x3)
')%_('
')%_('%choiceff(data=final, model=class(x1-x3 / sta), nsets=3, nalts=3,
')%_('          options=relative, beta=zero, init=final(keep=set), intiter=0)
')%_('
')%_('
')%_('%mktex(4 ** 6, n=32, seed=104)      /* choice design with restrictions      */
')%_('
')%_('%macro res;
')%_('   do i = 1 to nalts;
')%_('      do k = i + 1 to nalts;
')%_('         if all(x[i,] >= x[k,])     /* alt i dominates alt k                */
')%_('            then bad = bad + 1;
')%_('         if all(x[k,] >= x[i,])     /* alt k dominates alt i                */
')%_('            then bad = bad + 1;
')%_('         end;
')%_('      end;
')%_('   %mend;
')%_('
')%_('%choiceff(data=randomized,          /* candidate set of alternatives        */
')%_('          model=class(x1-x6 / sta), /* model with stdz orthogonal coding    */
')%_('          nsets=8,                  /* number of choice sets                */
')%_('          flags=4,                  /* 4 alternatives, generic candidates   */
')%_('          seed=104,                 /* random number seed                   */
')%_('          options=relative          /* display relative D-efficiency        */
')%_('                  resrep,           /* detailed report on restrictions      */
')%_('          restrictions=res,         /* name of the restrictions macro       */
')%_('          resvars=x1-x6,            /* variable names used in restrictions  */
')%_('          maxiter=1,                /* maximum number of designs to make    */
')%_('          bestout=desres1,          /* final choice design                  */
')%_('          beta=zero)                /* assumed beta vector, Ho: b=0         */
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktAllo Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb data=}{\it SAS-data-set}         & input SAS data set\\
')%_('{\ttb freq=}{\it variable}             & frequency variable\\
')%_('{\ttb nalts=}$n$                       & number of alternatives\\
')%_('{\ttb out=}{\it SAS-data-set}          & output SAS data set\\
')%_('{\ttb vars=}{\it variable-list}        & input variables\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktallo(help) /* show option list with ? or help */
')%_('
')%_('data allocs2;
')%_('   input Brand $ 1-8 Price $ Count Set;
')%_('   datalines;
')%_('.              .         0 1
')%_('Brand  1     $50       103 1
')%_('Brand  2     $75        58 1
')%_('Brand  3     $50       318 1
')%_('Brand  4    $100        99 1
')%_('Brand  5    $100        54 1
')%_('Brand  6    $100        83 1
')%_('Brand  7     $75        71 1
')%_('Brand  8     $75        58 1
')%_('Brand  9     $75       100 1
')%_('Brand 10     $50        56 1
')%_('.              .        10 2
')%_('Brand  1    $100        73 2
')%_('Brand  2    $100        76 2
')%_('Brand  3    $100       342 2
')%_('Brand  4     $50        55 2
')%_('Brand  5     $75        50 2
')%_('Brand  6    $100        77 2
')%_('Brand  7     $75        95 2
')%_('Brand  8    $100        71 2
')%_('Brand  9     $50        72 2
')%_('Brand 10    $100        79 2
')%_(';
')%_('
')%_('%mktallo(data=allocs2, out=allocs3, nalts=11,
')%_('         vars=set brand price, freq=Count)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktBal Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb list}                       & (positional) list of the numbers of levels\\
')%_('{\ttb init=}{\it SAS-data-set}    & initial input experimental design\\
')%_('{\ttb iter=}$n$                   & maximum iterations (designs to create)\\
')%_('{\ttb maxinititer=}$n$            & maximum initialization iterations \\
')%_('{\ttb maxiter=}$n$                & maximum iterations (designs to create)\\
')%_('{\ttb maxstarts=}$n$              & maximum number of random starts\\
')%_('{\ttb maxtries=}$n$               & times to try refining each factor\\
')%_('{\ttb n=}$n$                      & number of runs in the design\\
')%_('{\ttb options=noprint}            & suppress the final {\it D}-efficiency \\
')%_('{\ttb options=nosort}             & do not sort the final design \\
')%_('{\ttb options=oa}                 & an orthogonal array is sought \\
')%_('{\ttb options=progress}           & reports on macro progress \\
')%_('{\ttb options=sequential}         & factors are added but not refined \\
')%_('{\ttb out=}{\it SAS-data set}     & output experimental design\\
')%_('{\ttb seed=}$n$                   & random number seed\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktbal(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktbal(2 2 3 3 3, n=18, seed=151)
')%_('
')%_('
')%_('%mktbal(2 ** 23, n=24, options=oa progress, maxstarts=2000, seed=104)
')%_('
')%_('
')%_('%mktex(10 ** 4, n=100)
')%_('%mktbal(10 ** 4 2 2, n=100, init=design, options=oa progress,
')%_('        maxstarts=5000, seed=104, out=balanced)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktBIBD Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb b=}$b$                     & number of blocks (alias for {\ttb nsets=})\\
')%_('{\ttb group=}$n$                 & number of groups\\
')%_('{\ttb k=}$k$                     & block size (alias for {\ttb setsize=})\\
')%_('{\ttb nattrs=}$t$                & number of attributes (alias for {\ttb t=})\\
')%_('{\ttb nsets=}$b$                 & number of sets (alias for {\ttb b=}) \\
')%_('{\ttb optiter=}\noptott          & number of PROC OPTEX iterations\\
')%_('{\ttb options=position}          & optimizes position frequencies \\
')%_('{\ttb options=neighbor}          & nondirectional row-neighbor balance and position \\
')%_('{\ttb options=serial}            & directional row-neighbor balance and position \\
')%_('{\ttb out=}{\it SAS-data-set}    & output data set BIBD\\
')%_('{\ttb outf=}{\it SAS-data-set}   & output data set factorial design\\
')%_('{\ttb outi=}{\it SAS-data-set}   & output data set incidence matrix\\
')%_('{\ttb outs=}{\it SAS-data-set}   & output data set sorted design\\
')%_('{\ttb positer=}\noptott          & number of iterations position frequencies\\
')%_('{\ttb seed=}$n$                  & random number seed\\
')%_('{\ttb setsize=}$k$               & set size (alias for {\ttb k=}) \\
')%_('{\ttb t=}$t$                     & number of treatments (alias for {\ttb nattrs=})\\
')%_('{\ttb weights=}$n$ $n$           & position frequency badness weights\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktbibd(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktbibd(b=10, t=5, k=3, seed=104)
')%_('
')%_('
')%_('%mktbibd(nsets=10, nattrs=5, setsize=3, seed=104)
')%_('
')%_('
')%_('%mktbibd(b=14, t=7, k=4, options=neighbor, seed=104)
')%_('
')%_('
')%_('%mktbibd(b=14, t=7, k=4, options=serial, seed=361699)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktBlock Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb alt=}{\it variable}          & alternative number variable\\
')%_('{\ttb block=}{\it variable}        & block number variable\\
')%_('{\ttb data=}{\it SAS-data-set}     & either the choice or linear design\\
')%_('{\ttb factors=}{\it variable-list} & factors in the design\\
')%_('{\ttb id=}{\it variable-list}      & variables to copy to output data set\\
')%_('{\ttb initblock=}{\it variable}    & initial blocking variable\\
')%_('{\ttb iter=}$n$                    & times to try to block the design\\
')%_('{\ttb list=}$n$                    & list larger canonical correlations\\
')%_('{\ttb maxiter=}$n$                 & times to try to block the design\\
')%_('{\ttb nalts=}$n$                   & number of alternatives in choice set\\
')%_('{\ttb nblocks=}$n$                 & number of blocks to create\\
')%_('{\ttb next=}$n$                    & where to look for the next exchange\\
')%_('{\ttb options=nosort}              & do not sort the design into blocks \\
')%_('{\ttb out=}{\it SAS-data-set}      & output data set with block numbers\\
')%_('{\ttb outr=}{\it SAS-data-set}     & randomized output data set\\
')%_('{\ttb print=}{\it print-options}   & output display options\\
')%_('{\ttb ridge=}$n$                   & ridging factor\\
')%_('{\ttb seed=}$n$                    & random number seed\\
')%_('{\ttb set=}{\it variable}          & choice set number variable\\
')%_('{\ttb vars=}{\it variable-list}    & factors in the design\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktblock(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktex(3 ** 6, n=27)
')%_('
')%_('%mktblock(data=randomized, nblocks=3, factors=x1-x6, seed=377)
')%_('
')%_('
')%_('%mktex(3 ** 6, n=729)
')%_('
')%_('%mktroll(design=randomized, key=2 3, out=out)
')%_('
')%_('%choiceff(data=out, model=class(x1-x3), nsets=18, nalts=2,
')%_('          beta=zero, options=nodups, seed=151)
')%_('
')%_('%mktblock(data=best, nalts=2, nblocks=2, factors=x1-x3, seed=472)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktBSize Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb b=}{\it do-list}            & number of blocks (alias for {\ttb nsets=})\\
')%_('{\ttb k=}{\it do-list}            & block size (alias for {\ttb setsize=})\\
')%_('{\ttb maxreps=}$n$                & maximum number of replications\\
')%_('{\ttb nattrs=}{\it do-list}       & number of attributes (alias for {\ttb t=})\\
')%_('{\ttb nsets=}{\it do-list}        & number of sets (alias for {\ttb b=}) \\
')%_('{\ttb order=}{\it order-list}     & order of the loops\\
')%_('{\ttb options=nocheck}            & suppress checking $b$, $t$, and $k$ \\
')%_('{\ttb options=ubd}                & lifts the balance restriction on the design \\
')%_('{\ttb out=}{\it SAS-data-set}     & output data set design list\\
')%_('{\ttb setsize=}{\it do-list}      & set size (alias for {\ttb k=}) \\
')%_('{\ttb t=}{\it do-list}            & number of treatments (alias for {\ttb nattrs=})\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktbsize(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktbsize(t=12, k=4 to 8, b=12 to 30)
')%_('
')%_('
')%_('%mktbsize(nattrs=12, setsize=4 to 8, nsets=12 to 30)
')%_('
')%_('
')%_('%mktbsize(t=5 to 20, k=3 to t - 1, nsets=t to 30)
')%_('
')%_('
')%_('%mktbsize(nattrs=5 to 20, setsize=3 to t - 1, nsets=t to 30)
')%_('
')%_('
')%_('%mktbsize(t=20, k=6, options=ubd)
')%_('
')%_('
')%_('%mktbsize(nattrs=20, setsize=6, options=ubd)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktDes Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb big=}$n$                         & size of big candidate set\\
')%_('{\ttb cand=}{\it SAS-data-set}         & candidate design\\
')%_('{\ttb classopts=}{\it options}         & {\ttb class} statement options\\
')%_('{\ttb coding=}{\it name}               & {\ttb coding=} option\\
')%_('{\ttb examine=}$<$ I $>$   $<$ V $>$   & matrices that you want to examine\\
')%_('{\ttb facopts=}{\it options}           & PROC FACTEX statement options\\
')%_('{\ttb factors=}{\it factor-list}       & factors and levels for each factor\\
')%_('{\ttb generate=}{\it options}          & {\ttb generate} statement options\\
')%_('{\ttb interact=}{\it interaction-list} & interaction terms\\
')%_('{\ttb iter=}$n$                        & number of designs\\
')%_('{\ttb keep=}$n$                        & number of designs to keep\\
')%_('{\ttb maxiter=}$n$                     & number of designs\\
')%_('{\ttb method=}{\it name}               & search method\\
')%_('{\ttb n=}$n$ $|$ SATURATED             & number of runs\\
')%_('{\ttb nlev=}$n$                        & number of levels for pseudo-factors\\
')%_('{\ttb options=allcode}                 & shows all code \\
')%_('{\ttb options=check}                   & checks the {\ttb cand=} design \\
')%_('{\ttb options=nocode}                  & suppress the procedure code display \\
')%_('{\ttb otherfac=}{\it variable-list}    & other factors\\
')%_('{\ttb otherint=}{\it terms}            & multi-step interaction terms\\
')%_('{\ttb out=}{\it SAS-data-set}          & output experimental design\\
')%_('{\ttb procopts=}{\it options}          & PROC OPTEX statement options\\
')%_('{\ttb run=}{\it procedure-list}        & list of procedures that can be run\\
')%_('{\ttb seed=}$n$                        & random number seed\\
')%_('{\ttb size=}$n$ $|$ MIN                & candidate-set size\\
')%_('{\ttb step=}$n$                        & step number\\
')%_('{\ttb where=}{\it where-clause}        & \tts{where} clause\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktdes(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktdes(factors=x1-x2=2 x3-x5=3, n=18)
')%_('
')%_('
')%_('%mktdes(factors=x1-x3=2 x4-x6=3 x7-x9=5, n=30)
')%_('
')%_('
')%_('%mktdes(factors=x1-x3=2, n=30, run=factex,       step=1)
')%_('%mktdes(factors=x4-x6=3, n=30, run=factex,       step=2)
')%_('%mktdes(factors=x7-x9=5, n=30, run=factex optex, step=3)
')%_('
')%_('
')%_('%mktdes(factors=x1-x3=2 x4-x6=3(3) x7-x9=4, n=32, size=1024,
')%_('        interact=x1|x2|x3@2 x4*x7)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktDups Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb options}                         & (positional) binary options\\
')%_('{\ttb data=}{\it SAS-data-set}         & input choice design\\
')%_('{\ttb factors=}{\it variable-list}     & factors in the design\\
')%_('{\ttb nalts=}$n$                       & number of alternatives\\
')%_('{\ttb out=}{\it SAS-data-set}          & output data set\\
')%_('{\ttb outlist=}{\it SAS-data-set}      & output data set with duplicates\\
')%_('{\ttb vars=}{\it variable-list}        & factors in the design\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktdups(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktex(3 ** 3 2 ** 2, n=19, seed=513)
')%_('
')%_('%mktdups(linear, factors=x1-x5);
')%_('
')%_('
')%_('%mktex(2 ** 5, n=2**5)
')%_('
')%_('%mktlab(int=f1-f4, values=1 2)
')%_('
')%_('%choiceff(data=final, model=class(x1-x5), seed=109,
')%_('          nsets=42, flags=f1-f4, beta=zero);
')%_('
')%_('%mktdups(generic, data=best, factors=x1-x5, nalts=4, out=out)
')%_('
')%_('
')%_('%mktex(3 ** 9, n=27)
')%_('
')%_('data key;
')%_('   input (Brand x1-x3) ($) @@;
')%_('   datalines;
')%_('Acme x1 x2 x3 Ajax x4 x5 x6 Widgit x7 x8 x9
')%_(';
')%_('%mktroll(design=randomized, key=key, alt=brand, out=cand);
')%_('%choiceff(data=cand, model=class(brand x1-x3), seed=420,
')%_('          nsets=18, nalts=3, beta=zero);
')%_('%mktdups(branded, data=best, factors=brand x1-x3, nalts=3, out=out)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktEval Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb blocks=}{\it variable}       & blocking variable\\
')%_('{\ttb data=}{\it SAS-data-set}     & input data set with design\\
')%_('{\ttb factors=}{\it variable-list} & factors in the design\\
')%_('{\ttb format=}{\it format}         & format for canonical correlations\\
')%_('{\ttb freqs=}{\it frequency-list}  & frequencies to display\\
')%_('{\ttb list=}$n$                    & minimum canonical correlation to list\\
')%_('{\ttb outcb=}{\it SAS-data-set}    & within-block canonical correlations\\
')%_('{\ttb outcorr=}{\it SAS-data-set}  & canonical correlation matrix\\
')%_('{\ttb outfreq=}{\it SAS-data-set}  & frequencies\\
')%_('{\ttb outfsum=}{\it SAS-data-set}  & frequency summaries\\
')%_('{\ttb outlist=}{\it SAS-data-set}  & list of largest canonical correlations\\
')%_('{\ttb print=}{\it print-options}   & controls the display of the results\\
')%_('{\ttb vars=}{\it variable-list}    & factors in the design\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mkteval(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktex(2 ** 2 3 ** 3, n=18, seed=205)
')%_('
')%_('%mkteval;
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktEx Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb list}                          & (positional) list of the numbers of factor levels\\
')%_('{\ttb anneal=}\noptott               & starting probability for annealing\\
')%_('{\ttb annealfun=}{\it function}      & annealing probability function\\
')%_('{\ttb anniter=}\noptott              & first annealing iteration\\
')%_('{\ttb balance=}$n$                   & maximum level-frequency range\\
')%_('{\ttb big=}$n$ $<$ {\it choose} $>$  & size of big full-factorial design\\
')%_('{\ttb canditer=}\noptot              & iterations for OPTEX designs\\
')%_('{\ttb cat=}{\it SAS-data-set}        & input design catalog\\
')%_('{\ttb detfuzz=}$n$                   & determinants change increment\\
')%_('{\ttb examine=}$<$ I $>$ $<$ V $>$ $<${\it aliasing} $>$& matrices that you want to examine\\
')%_('{\ttb exchange=}$n$                  & number of factors to exchange\\
')%_('{\ttb fixed=}{\it variable}          & indicates runs that are fixed\\
')%_('{\ttb holdouts=}$n$                  & adds holdout observations\\
')%_('{\ttb imlopts=}{\it options}         & IML PROC statement options\\
')%_('{\ttb init=}{\it SAS-data-set}       & initial input experimental design\\
')%_('{\ttb interact=}{\it interaction-list} & interaction terms\\
')%_('{\ttb iter=}\noptott                 & maximum number of iterations\\
')%_('{\ttb levels=}{\it value}            & assigning final factor levels\\
')%_('{\ttb maxdesigns=}$n$                & maximum number of designs to make\\
')%_('{\ttb maxiter=}\noptott              & maximum number of iterations\\
')%_('{\ttb maxstages=}$n$                 & maximum number of algorithm stages\\
')%_('{\ttb maxtime=}\noptott              & approximate maximum run time\\
')%_('{\ttb mintry=}$n$                    & minimum number of rows to process\\
')%_('{\ttb mutate=}\noptott               & mutation probability\\
')%_('{\ttb mutiter=}\noptott              & first iteration to consider mutating\\
')%_('{\ttb n=}$n$                         & number of runs in the design\\
')%_('{\ttb options=accept}                & accept designs that violate restrictions \\
')%_('{\ttb options=check}                 & checks the efficiency of the {\ttb init=} design \\
')%_('{\ttb options=file}                  & renders the design to a file \\
')%_('{\ttb options=int}                   & add an intercept variable {\ttb x0} to the design \\
')%_('{\ttb options=justinit}              & stop processing after making the initial design \\
')%_('{\ttb options=largedesign}           & stop after {\ttb maxtime=} minutes have elapsed \\
')%_('{\ttb options=lineage}               & displays the lineage of the orthogonal array \\
')%_('{\ttb options=nodups}                & eliminates duplicate runs \\
')%_('{\ttb options=nofinal}               & skips displaying the final efficiency \\
')%_('{\ttb options=nohistory}             & does not display the iteration history \\
')%_('{\ttb options=nooadups}              & check for orthogonal array with duplicate runs \\
')%_('{\ttb options=noqc}                  & do not use the  SAS/QC product \\
')%_('{\ttb options=nosort}                & does not sort the design \\
')%_('{\ttb options=nox}                   & suppress the {\ttb x}$n$ scalars with restrictions \\
')%_('{\ttb options=quick}                 & {\ttb optiter=0}, {\ttb maxdesigns=2}, {\ttb unbalanced=0}, {\ttb tabiter=1} \\
')%_('{\ttb options=quickr}                & {\ttb optiter=0}, {\ttb maxdesigns=1}, {\ttb unbalanced=0}, {\ttb tabiter=0} \\
')%_('{\ttb options=quickt}                & {\ttb optiter=0}, {\ttb maxdesigns=1}, {\ttb unbalanced=0}, {\ttb tabiter=1} \\
')%_('{\ttb options=render}                & displays the design compactly in the SAS listing \\
')%_('{\ttb options=refine}                & with {\ttb init=}, never reinitializes \\
')%_('{\ttb options=resrep}                & reports on the progress of the restrictions \\
')%_('{\ttb options=+-}                    & renders --1 as -- and 1 as + in two-level factors \\
')%_('{\ttb options=3}                     & applies {\ttb options=+-} to 3-level factors \\
')%_('{\ttb options=512}                   & adds some designs in 512 runs \\
')%_('{\ttb optiter=}\noptot               & OPTEX iterations\\
')%_('{\ttb order=}{\it value}             & coordinate exchange column order\\
')%_('{\ttb out=}{\it SAS-data-set}        & output experimental design\\
')%_('{\ttb outall=}{\it SAS-data-set}     & output data set with all designs\\
')%_('{\ttb outeff=}{\it SAS-data-set}     & output data set with final efficiency\\
')%_('{\ttb outr=}{\it SAS-data-set}       & randomized output experimental design\\
')%_('{\ttb partial=}$n$                   & partial-profile design\\
')%_('{\ttb repeat=}{\it n1 n2 n3}         & times to iterate on a row \\
')%_('{\ttb reslist=}{\it list}            & constant matrix list\\
')%_('{\ttb resmac=}{\it macro-name}       & constant matrix creation macro\\
')%_('{\ttb restrictions=}{\it macro-name} & restrictions macro\\
')%_('{\ttb ridge=}$n$                     & ridging factor\\
')%_('{\ttb seed=}$n$                      & random number seed\\
')%_('{\ttb stopearly=}$n$                 & the macro can stop early\\
')%_('{\ttb tabiter=}\noptot               & design table initialization iterations\\
')%_('{\ttb tabsize=}$n$                   & orthogonal array size\\
')%_('{\ttb target=}$n$                    & target efficiency criterion\\
')%_('{\ttb unbalanced=}\noptot            & unbalance initial design iterations\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktex(help) /* show option list with ? or help */
')%_('
')%_('%mktex(2 3 ** 7, n=18)
')%_('
')%_('%mktex(n=104, options=int)
')%_('
')%_('%mktex(2 ** 5  3 ** 4  5 5 5  6 6, n=60, seed=104, maxtime=.3 .3 .3)
')%_('
')%_('%macro resmac;
')%_('   client    = {1.29 1.69 2.09 .}[x1];
')%_('   extension = {1.39 1.89 2.39 .}[x2];
')%_('   regional  = {1.99 2.49      .}[x5];
')%_('   private   = {1.49 2.29      .}[x6];
')%_('   national  = {1.99 2.39      .}[x8];
')%_('   navail    = (client  ^= .) + (extension ^= .) + (regional ^= .) +
')%_('               (private ^= .) + (national  ^= .);
')%_('   if (navail < 2) | (navail > 4) then bad = abs(navail - 3);
')%_('   else                                bad = 0;
')%_('   %mend;
')%_('
')%_('%mktex( 4 4 2 2 3 3 2 3, n=26, interact=x2*x3 x2*x4 x3*x4 x6*x7,
')%_('        restrictions=resmac, seed=377 )
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktKey Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb list}                       & (positional) variable list or n rows and n columns \\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktkey(help) /* show option list with ? or help */
')%_('
')%_('%mktkey(3 5)
')%_('
')%_('%mktkey(5 10 t)
')%_('
')%_('%mktkey(x1-x20)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktLab Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb cfill=}{\it character-string}    & character fill value\\
')%_('{\ttb data=}{\it SAS-data-set}         & input design data set\\
')%_('{\ttb dolist=}{\it do-list}            & new values using a do-list syntax\\
')%_('{\ttb int=}{\it variable-list}         & name of an intercept variable\\
')%_('{\ttb key=}{\it SAS-data-set}          & key data set\\
')%_('{\ttb labels=}{\it macro-name}         & macro that provides labels and formats\\
')%_('{\ttb nfill=}{\it number}              & numeric fill value\\
')%_('{\ttb options=noprint}                 & suppress the display of the variable mappings \\
')%_('{\ttb out=}{\it SAS-data-set}          & output data set with recoded design\\
')%_('{\ttb prefix=}{\it variable-prefix}    & prefix for naming variables\\
')%_('{\ttb statements=}{\it SAS-code}       & add extra statements\\
')%_('{\ttb values=}{\it value-list}         & the new values for all of the variables\\
')%_('{\ttb vars=}{\it variable-list}        & list of variable names\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktlab(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktex(n=12, options=nosort)
')%_('
')%_('%mktlab(data=design, values=-1 1, int=Had0, prefix=Had);
')%_('
')%_('
')%_('%mktex(n=18, seed=17)
')%_('
')%_('%mktblock(nblocks=2, factors=x1-x4)
')%_('
')%_('data key;
')%_('   input Brand $ Price Size;
')%_('   format price dollar5.2;
')%_('   datalines;
')%_('Acme 1.49   6
')%_('Apex 1.79   8
')%_('.    1.99  12
')%_(';
')%_('
')%_('%mktlab(data=blocked, key=key)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktMDiff Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb layout}                       & (positional) choice data set structure \\
')%_('{\ttb attrs=}{\it macro-variable}   & macro variable with attribute descriptions\\
')%_('{\ttb b=}$b$                        & number of sets (alias for {\ttb nsets=})\\
')%_('{\ttb classopts=}{\it options-list} & class variable options \\
')%_('{\ttb data=}{\it SAS-data-set}      & input data set \\
')%_('{\ttb design=}{\it SAS-data-set}    & design data set \\
')%_('{\ttb group=}$k$                    & number of groups in the {\ttb design=} data set \\
')%_('{\ttb k=}$k$                        & set size (alias for {\ttb setsize=})\\
')%_('{\ttb nattrs=}$t$                   & number of attributes (alias for {\ttb t=})\\
')%_('{\ttb nsets=}$b$                    & number of sets (alias for {\ttb b=}) \\
')%_('{\ttb options=nocode}               & just create the OUT= data set \\
')%_('{\ttb options=noanalysis}           & just create the OUT= and OUTCODED= data sets \\
')%_('{\ttb options=nosort}               & do not sort the parameter estimates table \\
')%_('{\ttb options=nodrop}               & do not drop the variable or parameter column \\
')%_('{\ttb out=}{\it SAS-data-set}       & merged data and design data set \\
')%_('{\ttb outcoded=}{\it SAS-data-set}  & coded data set \\
')%_('{\ttb outparm=}{\it SAS-data-set}   & parameter estimates data set \\
')%_('{\ttb rescale=}{\it value}          & table of rescaled parameter estimates \\
')%_('{\ttb setsize=}$k$                  & set size (alias for {\ttb k=}) \\
')%_('{\ttb t=}$t$                        & number of attributes or messages (alias for {\ttb nattrs=})\\
')%_('{\ttb vars=}{\it variable-list}     & {\ttb data=} data set variables \\
')%_('\end{longtable}
')%_('
')%_('\enlargethispage{3\baselineskip}
')%_('\codestart
')%_('%mktmdiff(help) /* show option list with ? or help */
')%_('
')%_('%mktbibd(t=9, k=5, b=18, seed=377, out=sasuser.bibd)
')%_('
')%_('title "Best Worst Example with Cell Phone Attributes";
')%_('data bestworst;
')%_('   input Sub $ @4 (b1-b18 w1-w18) (1.);
')%_('   datalines;
')%_(' 1 188661884399349653941955342212935494
')%_(' 2 765358873891388493922673644336595554
')%_(' 3 782126282892848564995993447213935655
')%_(' 4 481363264246399187162125415351281453
')%_(' 5 787168863811878175225995382352235293
')%_(' 6 787658867891878667495965442313345453
')%_(' 7 788171867736888187465395344393344453
')%_(' 8 788771867711888687445353445353335254
')%_(' 9 188778887896878687425323443242344454
')%_('10 788778887816888687445353444343344454
')%_(';
')%_('\end{Verbatim}
')%_('
')%_('\codebreak
')%_('%let attributes=Camera,Flip,Hands Free,Games,Internet
')%_(',Free Replacement,Battery Life,Large Letters,Applications;
')%_('%phchoice( on )
')%_('
')%_('%mktmdiff(bw, b=18, t=9, k=5, attrs=attributes,
')%_('          data=bestworst, design=sasuser.bibd)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktMerge Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb blocks=}1 $|$ {\it variable}     & blocking variable\\
')%_('{\ttb data=}{\it SAS-data-set}         & input SAS data set\\
')%_('{\ttb design=}{\it SAS-data-set}       & input SAS choice design data set\\
')%_('{\ttb nalts=}$n$                       & number of alternatives\\
')%_('{\ttb nsets=}$n$                       & number of choice sets\\
')%_('{\ttb out=}{\it SAS-data-set}          & output SAS data set\\
')%_('{\ttb setvars=}{\it variable-list}     & variables with the data\\
')%_('{\ttb statements=}{\it SAS-statements} & additional statements\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktmerge(help) /* show option list with ? or help */
')%_('
')%_('%mktex(3 ** 4, n=18, seed=104)
')%_('
')%_('data results;
')%_('   input Subj (choose1-choose18) (1.) @@;
')%_('   datalines;
')%_('  1 333542334333314443  2 333212344333333345  3 333212333333313333
')%_('  4 133242144334414453  5 335242134333513443  6 333242234333314443
')%_('  7 333432334332323443  8 333242234334414443  9 333432331352313343
')%_(';
')%_('
')%_('data key;
')%_('   input Brand $ & Price $;
')%_('   datalines;
')%_('Brand 1  x1
')%_('Brand 2  x2
')%_('Brand 3  x3
')%_('Brand 4  x4
')%_('Another  .
')%_(';
')%_('
')%_('%mktroll(design=randomized, key=key, alt=brand, out=rolled)
')%_('
')%_('%mktmerge(design=rolled, data=results, out=res2,
')%_('          nsets=18, nalts=5, setvars=choose1-choose18)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktOrth Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb filter=}$n$                 & extra filtering of the design catalog\\
')%_('{\ttb maxn=}$n$                   & maximum number of runs of interest\\
')%_('{\ttb maxlev=}$n$                 & maximum number of levels\\
')%_('{\ttb options=lineage}            & construct the design lineage \\
')%_('{\ttb options=mktex}              & the macro is being called from the \mktex macro \\
')%_('{\ttb options=mktruns}            & the macro is being called from the \mktruns macro \\
')%_('{\ttb options=parent}             & lists only parent designs \\
')%_('{\ttb options=dups}               & suppress duplicate and inferior design filtering \\
')%_('{\ttb options=512}                & adds some designs in 512 runs \\
')%_('{\ttb outall=}{\it SAS-data-set}  & output data set with all designs\\
')%_('{\ttb outcat=}{\it SAS-data-set}  & design catalog data set\\
')%_('{\ttb outlev=}{\it SAS-data-set}  & output data set with the list of levels\\
')%_('{\ttb range=}{\it range-specification} & number of runs of interest\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktorth(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%mktorth(range=n eq 12 or n eq 18);
')%_('
')%_('proc print; run;
')%_('
')%_('
')%_('%mktorth(range=n=72)
')%_('
')%_('proc print data=mktdeslev(where=(x2 ge 5 and x3 ge 5 and x4 and x6));
')%_('   var n design reference;
')%_('   run;
')%_('
')%_('
')%_('%mktorth(maxn=100, maxlev=6)
')%_('
')%_('
')%_('%mktorth(range=n=36, options=lineage)
')%_('proc print; run;
')%_('
')%_('
')%_('%mktorth(range=n=36, options=parent)
')%_('proc print; run;
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktPPro Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb ibd=}{\it SAS-data-set}    & incomplete block design \\
')%_('{\ttb design=}{\it SAS-data-set} & orthogonal array \\
')%_('{\ttb out=}{\it SAS-data-set}    & output partial-profile design \\
')%_('{\ttb print=}{\it print-options} & output display options \\
')%_('{\ttb x=}{\it SAS-data-set}      & incomplete block design incidence matrix \\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktppro(help) /* show option list with ? or help */
')%_('
')%_('%mktex(2 4 2 2 2, n=8)
')%_('
')%_('data design; set design; drop x2; run;
')%_('
')%_('%mktbibd(b=20, nattrs=16, setsize=4, seed=104, out=b)
')%_('
')%_('%mktppro(ibd=b, print=f p)
')%_('
')%_('%choiceff(data=chdes, model=class(x1-x16 / sta), nsets=80, nalts=2,
')%_('          options=relative, beta=zero, init=chdes, initvars=x1-x16)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktRoll Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb alt=}{\it variable}              & variable with name of each alternative\\
')%_('{\ttb design=}{\it SAS-data-set}       & input SAS data set\\
')%_('{\ttb keep=}{\it variable-list}        & factors to keep\\
')%_('{\ttb key=}{\it SAS-data-set}          & key data set name \\
')%_('{\ttb key=}{\it rows columns} $<${\ttb t}$>$ & key data set description\\
')%_('{\ttb options=nowarn}                  & suppress the variables not mentioned warning \\
')%_('{\ttb out=}{\it SAS-data-set}          & output SAS data set\\
')%_('{\ttb set=}{\it variable}              & choice set number variable\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktroll(help) /* show option list with ? or help */
')%_('
')%_('%mktex(3 ** 5, n=12)
')%_('
')%_('data key;
')%_('   input (Brand Price) ($);
')%_('   datalines;
')%_('A x1
')%_('B x2
')%_('C x3
')%_('D x4
')%_('E x5
')%_(';
')%_('
')%_('%mktroll(design=randomized, key=key, out=sasuser.design, alt=brand)
')%_('
')%_('proc print; by set; id set; run;
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf MktRuns Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb list}                       & (positional) numbers of levels of all the factors\\
')%_('{\ttb interact=}{\it interaction-list} & interaction terms \\
')%_('{\ttb max=}$n \;\; < m > \;\; < {\rm f} >$ & largest design sizes to try\\
')%_('{\ttb n=}$n$                      & design size to evaluate\\
')%_('{\ttb maxlev=}$n$                 & maximum number of levels\\
')%_('{\ttb maxoa=}$n$                  & maximum number of orthogonal arrays\\
')%_('{\ttb options=justparse}          & used by other {\ttb Mkt} macros to parse the list \\
')%_('{\ttb options=multiple}           & allow terms to be counted multiple times \\
')%_('{\ttb options=multiple2}          & {\ttb option=multiple} and more detailed output \\
')%_('{\ttb options=noprint}            & suppress the display of all output \\
')%_('{\ttb options=nosat}              & suppress the saturated design from the design list \\
')%_('{\ttb options=source}             & displays source of numbers in design sizes \\
')%_('{\ttb options=512}                & adds some designs in 512 runs \\
')%_('{\ttb out=}{\it SAS-data-set}     & data set with the suggested sizes\\
')%_('{\ttb outorth=}{\it SAS-data-set} & data set with orthogonal array list\\
')%_('{\ttb toobig=}$n$                 & specifies problem that is too big\\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%mktruns(help) /* show option list with ? or help */
')%_('
')%_('%mktruns(2 2 2 3 3 3 3)
')%_('
')%_('%mktruns(2**10 3**3, interact=11|12|13@2)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf Paint Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb values=}{\it do-list}               & input data values \\
')%_('{\ttb var=}{\it variable-name}            & input variable \\
')%_('{\ttb data=}{\it SAS-data-set}            & input SAS data set \\
')%_('{\ttb out=}{\it SAS-data-set}             & output SAS data set \\
')%_('{\ttb macro=}{\it macro-name}             & SAS macro name \\
')%_('{\ttb colors=}$<${\it color-list}$>$ $<${\it data-value-list}$>$ & colors list \\
')%_('{\ttb level=}{\it measurement-level}      & measurement level of the data \\
')%_('{\ttb symbols=}{\it symbols-list}         & list of symbols  \\
')%_('{\ttb format=}{\it SAS-format}            & nominal and ordinal variable format\\
')%_('{\ttb order=}{\it summary-order}          & nominal and ordinal variable order\\
')%_('{\ttb select=}{\it n}                     & selection state \\
')%_('{\ttb show=}{\it n}                       & show/hide state \\
')%_('{\ttb include=}{\it n}                    & include/exclude state \\
')%_('{\ttb label=}{\it n}                      & label/unlabel state \\
')%_('{\ttb rgbround=}{\it rounding-list}       & RGB value rounding instructions \\
')%_('{\ttb missing=}{\it missing-specification}& missing value handling \\
')%_('{\ttb debug=}{\it debug-string}           & debugging information to display \\
')%_('\end{longtable}
')%_('
')%_('\vspace{-\baselineskip}
')%_('\enlargethispage{3\baselineskip}
')%_('\codestart
')%_('%paint(help) /* show option list with ? or help */
')%_('
')%_('data cars(drop=i);
')%_('   do i = 1 to 7;
')%_('      input (MPG Reliability Acceleration Braking Handling Ride
')%_('             Visibility Comfort Quiet Cargo) (1.) +1 @;
')%_('      output;
')%_('      end;
')%_('   datalines;
')%_('3334444544 2434453555 2354353545 3244443424 2354553543 3232423224 4145555422
')%_('2254541241 2445353555 5335425223 4155555525 2153542242 3333444544 3253353544
')%_(';
')%_('
')%_('%paint(values=-1 to 1 by 0.05, macro=setstyle,
')%_('       colors=CXEEEEEE red magenta blue cyan white cyan blue magenta red CXEEEEEE
')%_('              -1 -0.99 -0.75 -0.5 -0.25 0 0.25 0.5 0.75 0.99 1)
')%_('
')%_('proc template;
')%_('   edit Base.Corr.StackedMatrix;
')%_('      column (RowName RowLabel) (Matrix) * (Matrix2);
')%_('      edit matrix;    %setstyle(backgroundcolor)    end;
')%_('      end;
')%_('   run;
')%_('
')%_('ods html body="corr.html" style=statistical;
')%_('proc corr data=cars noprob;    run;
')%_('ods html close;
')%_('proc template;   delete Base.Corr.StackedMatrix;   run;
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf PHChoice Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb onoff} & (positional) {\ttb on}, {\ttb off}, or {\ttb expb} \\
')%_('{\ttb column} & (positional) list of columns \\
')%_('\end{longtable}
')%_('
')%_('\codestart
')%_('%phchoice(help) /* show option list with ? or help */
')%_('
')%_('
')%_('%phchoice(on)
')%_('
')%_('
')%_('%phchoice(off)
')%_('
')%_('
')%_('%phchoice(expb)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('
')%_('\newpage
')%_('\begin{center}\Huge {\bf PlotIt Macro Options}\end{center}
')%_('
')%_('\begin{longtable}{ll}
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endfirsthead
')%_('\hline
')%_('Option & Description \\
')%_('\hline
')%_('\endhead
')%_('\hline
')%_('\endfoot
')%_('{\ttb adjust1=}{\it SAS-statements} & adjust the preprocessing data set\\
')%_('{\ttb adjust2=}{\it SAS-statements} & includes statements with PROC PLOT\\
')%_('{\ttb adjust3=}{\it SAS-statements} & extra statements for the final DATA step\\
')%_('{\ttb adjust4=}{\it SAS-statements} & extra statements for the final DATA step\\
')%_('{\ttb adjust5=}{\it SAS-statements} & extra statements for the final DATA step\\
')%_('{\ttb antiidea=}$n$                 & eliminates PREFMAP anti-ideal points\\
')%_('{\ttb blue=}{\it expression}        & blue part of RGB colors\\
')%_('{\ttb bright=}$n$                   & generates random label colors\\
')%_('{\ttb britypes=}{\it type}          & types to which {\ttb bright=} applies\\
')%_('{\ttb cframe=}{\it color}           & color of background within the frame\\
')%_('{\ttb cirsegs=}$n$                  & circle smoothness parameter\\
')%_('{\ttb color=}{\it color}            & default color\\
')%_('{\ttb colors=}{\it colors-list}     & default label and symbol color list\\
')%_('{\ttb cursegs=}$n$                  & number of segments in a curve\\
')%_('{\ttb curvecol=}{\it color}         & color of curve\\
')%_('{\ttb data=}{\it SAS-data-set}      & input data set\\
')%_('{\ttb datatype=}{\it data-type}     & data analysis that generated data set\\
')%_('{\ttb debug=}{\it values}           & debugging output\\
')%_('{\ttb excolors=}{\it color-list}    & excludes from the Annotate data set\\
')%_('{\ttb extend=}{\it axis-extensions} & extend the $x$ and $y$ axes\\
')%_('{\ttb extraobs=}{\it SAS-data-set}  & extra observations data set\\
')%_('{\ttb exttypes=}{\it type}          & types for {\ttb extraobs=} data set\\
')%_('{\ttb filepref=}{\it prefix}        & file name prefix\\
')%_('{\ttb font=}{\it font}              & default font\\
')%_('{\ttb framecol=}{\it color}         & color of frame\\
')%_('{\ttb gdesc=}{\it description}      & catalog description\\
')%_('{\ttb gname=}{\it name}             & catalog entry\\
')%_('{\ttb gopplot=}{\it goptions}       & {\ttb goptions} for plotting to screen\\
')%_('{\ttb gopprint=}{\it goptions}      & {\ttb goptions} for printing\\
')%_('{\ttb gopts2=}{\it goptions}        & {\ttb goptions} that are always used\\
')%_('{\ttb gopts=}{\it goptions}         & additional {\ttb goptions}\\
')%_('{\ttb gout=}{\it catalog}           & {\ttb proc ganno gout=} catalog\\
')%_('{\ttb green=}{\it expression}       & green part of RGB colors\\
')%_('{\ttb hminor=}$n$ $|$ {\it do-list} & horizontal axis minor tick marks\\
')%_('{\ttb hnobs=}$n$                    & horizontal observations for contour plots\\
')%_('{\ttb hpos=}$n$                     & horizontal positions in graphics area\\
')%_('{\ttb href=}{\it do-list}           & horizontal reference lines\\
')%_('{\ttb hsize=}$n$                    & horizontal graphics area size\\
')%_('{\ttb inc=}$n$                      & {\ttb haxis=by inc}, {\ttb vaxis=by inc}\\
')%_('{\ttb interpol=}{\it method}        & axis interpolation method\\
')%_('{\ttb labcol=}{\it label-colors}    & colors for the point labels\\
')%_('{\ttb label=}{\it label-statement}  & {\ttb label} statement\\
')%_('{\ttb labelcol=}{\it color}         & color of variable labels\\
')%_('{\ttb labelvar=}{\it label-variable}& point label variable\\
')%_('{\ttb labfont=}{\it label-fonts}    & fonts for the point labels\\
')%_('{\ttb labsize=}{\it label-sizes}    & sizes for the point labels\\
')%_('{\ttb ls=}$n$                       & how line sizes are generated\\
')%_('{\ttb lsinc=}$n$                    & increment to line size\\
')%_('{\ttb lsizes=}{\it number-list}     & line sizes (thicknesses)\\
')%_('{\ttb makefit=}$n$                  & proportion of graphics window to use\\
')%_('{\ttb maxiter=}$n$                  & maximum number of iterations\\
')%_('{\ttb maxokpen=}$n$                 & maximum acceptable penalty sum\\
')%_('{\ttb method=}{\it value}           & where to send the plot\\
')%_('{\ttb monochro=}{\it color}         & overrides all other colors\\
')%_('{\ttb nknots=}$n$                   & number of knots option\\
')%_('{\ttb offset=}$n$                   & move symbols for coincident points\\
')%_('{\ttb options=border}               & draws a border box \\
')%_('{\ttb options=close}                & close up the border box and the axes \\
')%_('{\ttb options=diag}                 & draws a diagonal reference line \\
')%_('{\ttb options=expand}               & expands the plot to fill the window \\
')%_('{\ttb options=noback}               & do not set the frame color \\
')%_('{\ttb options=nocenter}             & do not center \\
')%_('{\ttb options=noclip}               & do not clip \\
')%_('{\ttb options=nocode}               & suppress the PROC PLOT and {\ttb goptions} statements \\
')%_('{\ttb options=nodelete}             & do not delete intermediate data sets \\
')%_('{\ttb options=nohistory}            & suppress the iteration history table \\
')%_('{\ttb options=nolegend}             & suppress the display of the legends \\
')%_('{\ttb options=noprint}              & {\ttb nolegend}, {\ttb nocode}, and {\ttb nohistory} \\
')%_('{\ttb options=square}               & the same ticks for both axes and a square plot \\
')%_('{\ttb options=textline}             & lines overwrite text \\
')%_('{\ttb out=}{\it SAS-data-set}       & output Annotate data set\\
')%_('{\ttb outward=}{\ttb none} $|$ {\it char} & PLOT statement {\ttb outward=}\\
')%_('{\ttb paint=}{\it interpolation}    & color interpolation\\
')%_('{\ttb place=}{\it placement}        & generates a {\ttb placement=} option\\
')%_('{\ttb plotopts=}{\it options}       & PLOT statement options\\
')%_('{\ttb plotvars=}{\it variable-list} & {\it y}-axis and {\it x}-axis variables\\
')%_('{\ttb post=}{\it filename}          & graphics stream file name\\
')%_('{\ttb preproc=}{\it SAS-data-set}   & preprocessed {\ttb data=} data set\\
')%_('{\ttb procopts=}{\it options}       & PROC PLOT statement options\\
')%_('{\ttb ps=}$n$                       & page size\\
')%_('{\ttb radii=}{\it do-list}          & radii of circles\\
')%_('{\ttb red=}{\it expression}         & red part of RGB colors\\
')%_('{\ttb regdat=}{\it SAS-data-set}    & intermediate regression results data set\\
')%_('{\ttb regopts=}{\it options}        & regression curve fitting options\\
')%_('{\ttb regprint=}{\it regression-option} & regression options\\
')%_('{\ttb rgbround=}{\it RGB-rounding}  & {\ttb paint=} rounding factors\\
')%_('{\ttb rgbtypes=}{\it type}          & types to which {\ttb paint=} and RGB options apply\\
')%_('{\ttb style=}A $|$ B                & obsolete option not supported \\
')%_('{\ttb symbols=}{\it symbol-list}    & plotting symbols\\
')%_('{\ttb symcol=}{\it symbol-colors}   & colors of the symbols\\
')%_('{\ttb symfont=}{\it symbol fonts}   & symbol fonts\\
')%_('{\ttb symlen=}$n$                   & length of the symbols\\
')%_('{\ttb symsize=}{\it symbol-sizes}   & sizes of symbols\\
')%_('{\ttb symtype=}{\it symbol-types}   & types of symbols\\
')%_('{\ttb symvar=}{\it symbol-variable} & plotting symbol variable\\
')%_('{\ttb tempdat1=}{\it SAS-data-set}  & intermediate results data set\\
')%_('{\ttb tempdat2=}{\it SAS-data-set}  & intermediate results data set\\
')%_('{\ttb tempdat3=}{\it SAS-data-set}  & intermediate results data set\\
')%_('{\ttb tempdat4=}{\it SAS-data-set}  & intermediate results data set\\
')%_('{\ttb tempdat5=}{\it SAS-data-set}  & intermediate results data set\\
')%_('{\ttb tempdat6=}{\it SAS-data-set}  & intermediate results data set\\
')%_('{\ttb tickaxes=}{\it axis-string}   & axes to draw tick marks\\
')%_('{\ttb tickcol=}{\it color}          & color of ticks\\
')%_('{\ttb tickfor=}{\it format}         & tick format used by {\ttb interpol=tick}\\
')%_('{\ttb ticklen=}$n$                  & length of tick mark in horizontal cells\\
')%_('{\ttb titlecol=}{\it color}         & color of title\\
')%_('{\ttb tsize=}$n$                    & default text size\\
')%_('{\ttb types=}{\it observation-types}& observations types\\
')%_('{\ttb typevar=}{\it variable}       & observation types variable\\
')%_('{\ttb unit=}{\ttb in} $|$ {\ttb cm} & {\ttb hsize=} and {\ttb vsize=} unit\\
')%_('{\ttb vechead=}{\it vector-head-size} & how to draw vector heads\\
')%_('{\ttb vminor=}$n$ $|$ {\it do-list} & vertical axis minor tick marks\\
')%_('{\ttb vnobs=}$n$                    & vertical observations for contour plots\\
')%_('{\ttb vpos=}$n$                     & vertical positions in graphics area\\
')%_('{\ttb vref=}{\it do-list}           & vertical reference lines\\
')%_('{\ttb vsize=}$n$                    & vertical graphics area size\\
')%_('{\ttb vtoh=}$n$                     & PROC PLOT {\ttb vtoh=} option\\
')%_('{\ttb xmax=}$n$                     & maximum horizontal graphics area size\\
')%_('{\ttb ymax=}$n$                     & maximum vertical graphics area size\\
')%_('\end{longtable}
')%_('
')%_('\enlargethispage{3\baselineskip}
')%_('\codestart
')%_('%plotit(help) /* show option list with ? or help */
')%_('
')%_('data smoke;
')%_('   length Employee $ 20;
')%_('   input None Light Medium Heavy @13 Employee $ & Weight No Yes;
')%_('   label none    = "Nonsmoker"       light   = "Light Smoker"
')%_('         medium  = "Medium Smoker"   heavy   = "Heavy Smoker"
')%_('         no      = "Nondrinker"      yes     = "Drinker";
')%_('   datalines;
')%_(' 4  2  3  2 Senior Manager   1  0 11
')%_(' 4  3  7  4 Junior Manager   1  1 17
')%_('25 10 12  4 Senior Employee  1  5 46
')%_('18 24 33 13 Junior Employee  1 10 78
')%_('10  6  7  2 Admin            1  7 18
')%_('42 29 20  9 Nation          -1  .  .
')%_(';
')%_('\end{Verbatim}
')%_('
')%_('\codebreak
')%_('proc corresp outc=c;
')%_('   var    none -- heavy no yes;
')%_('   id     employee;
')%_('   weight weight;
')%_('   supvar no yes;
')%_('   run;
')%_('
')%_('%plotit(data=c, datatype=corresp)
')%_('\end{Verbatim}
')%_('\codestop
')%_('
')%_('\end{document}
')

return;

   doit:
   if opt =: '\begin{center' then do;
      macro = lowcase(scan(opt, 3, ' '));
      code  = 0;
      end;
   else if opt = '\codestart' then code = 1;
   else if opt = '\codestop'  then code = 0;
   if code and macro = "&macro" then do;
      if opt =: '\' then put /;
      else do;
         k = length(opt) - length(left(opt)) + 1;
         put @k opt;
         end;
      end;
   else do;
      if not (opt =: '{\ttb' and macro = "&macro") then return;
      opt = translate(opt, '  ', '$}');
      do i = 1 to 100 until(s = 0);
         s = 0;
         k = index(opt, '\tts' ); s + k; if k then substr(opt, k, 5) = ' ';
         k = index(opt, '\tts' ); s + k; if k then substr(opt, k, 5) = ' ';
         k = index(opt, '{\ttb'); s + k; if k then substr(opt, k, 5) = ' ';
         k = index(opt, '{\it' ); s + k; if k then substr(opt, k, 4) = ' ';
         k = index(opt, '{\rm' ); s + k; if k then substr(opt, k, 4) = ' ';
         k = index(opt, '\;'   ); s + k; if k then substr(opt, k, 2) = ' ';
         k = index(opt, '\\'   ); s + k; if k then substr(opt, k, 2) = ' ';
         k = index(opt, '< '   ); s + k; if k then substr(opt, k, 2) = ' <';
         k = index(opt, ' >'   ); s + k; if k then substr(opt, k, 2) = '> ';
         k = index(opt, ' ,'   ); s + k; if k then substr(opt, k, 2) = ', ';
         k = index(opt, ' )'   ); s + k; if k then substr(opt, k, 2) = ') ';
         k = index(opt, '--'   ); s + k; if k then substr(opt, k, 2) = '- ';
         k = index(opt, 'D -e' ); s + k; if k then substr(opt, k, 4) = ' D-e';
         k = index(opt, '\mkt' ); s + k; if k then substr(opt, k, 4) = ' mkt';
         end;
      opt = left(compbl(opt));
      k   = index(opt, '= ');
      if k then opt = substr(opt, 1, k) || substr(opt, k + 2);
      k   = index(opt, '&');
      val = substr(opt, k + 2);
      opt = substr(opt, 1, k - 1);
      k   = index(opt, '\noptott');
      if k then substr(opt, k) = 'n1 <n2 <n3>>';
      k   = index(opt, '\noptot');
      if k then substr(opt, k) = 'n1 <n2>';
      put opt $ 1-30 val $;
      end;
   return;
   run;

%mend;
