 /****************************************************************/
 /*          S A S   A U T O C A L L   L I B R A R Y             */
 /*                                                              */
 /*    NAME: MktLab                                              */
 /*   TITLE: Label Experimental Designs                          */
 /* PRODUCT: STAT                                                */
 /*  SYSTEM: ALL                                                 */
 /*    KEYS: marketing research, designs                         */
 /*   PROCS:                                                     */
 /*    DATA:                                                     */
 /* SUPPORT: saswfk                      UPDATE:  25Jul2010      */
 /*     REF: See "Marketing Research Methods in SAS" at the      */
 /*          following URL for more information on this macro.   */
 /*                                                              */
 /* http://support.sas.com/techsup/tnote/tnote_stat.html#market  */
 /*                                                              */
 /*    MISC: The following SAS products are required to run      */
 /*          this macro: BASE.                                   */
 /*                                                              */
 /*          This macro creates global macro variables: _MktVer  */
 /****************************************************************/

 /*---------------------------------------------------------------------

The macro MktLab is used to process an experimental design, usually
created by the MktEx macro, and assign the final variable names and
levels.

For example, say you used the MktEx macro to create a design with 11
two-level factors (with default levels of 1 and 2).

   %mktex(n=12, options=nosort)
   proc print noobs; run;

    x1    x2    x3    x4    x5    x6    x7    x8    x9    x10    x11

     1     1     1     1     1     1     1     1     1     1      1
     1     2     2     2     1     1     2     1     1     2      2
     2     2     1     1     2     1     2     2     1     2      1
     1     1     1     2     1     2     2     2     2     2      1
     1     2     2     2     2     1     1     2     2     1      1
     1     2     1     1     2     2     2     1     2     1      2
     2     2     1     2     1     2     1     2     1     1      2
     2     1     1     2     2     1     1     1     2     2      2
     2     1     2     2     2     2     2     1     1     1      1
     1     1     2     1     2     2     1     2     1     2      2
     2     2     2     1     1     2     1     1     2     2      1
     2     1     2     1     1     1     2     2     2     1      2

The MktLab macro can be used to assign levels of -1 and 1, add an
intercept, and change the variable name prefixes from "x" to "Had".
This creates a Hadamard matrix.

   %mktlab(data=design, values=-1 1, int=Had0, prefix=Had);
   proc print noobs; run;

Here is the resulting Hadamard matrix:

Had0  Had1  Had2  Had3  Had4  Had5  Had6  Had7  Had8  Had9  Had10  Had11

  1    -1    -1    -1    -1    -1    -1    -1    -1    -1     -1     -1
  1    -1     1     1     1    -1    -1     1    -1    -1      1      1
  1     1     1    -1    -1     1    -1     1     1    -1      1     -1
  1    -1    -1    -1     1    -1     1     1     1     1      1     -1
  1    -1     1     1     1     1    -1    -1     1     1     -1     -1
  1    -1     1    -1    -1     1     1     1    -1     1     -1      1
  1     1     1    -1     1    -1     1    -1     1    -1     -1      1
  1     1    -1    -1     1     1    -1    -1    -1     1      1      1
  1     1    -1     1     1     1     1     1    -1    -1     -1     -1
  1    -1    -1     1    -1     1     1    -1     1    -1      1      1
  1     1     1     1    -1    -1     1    -1    -1     1      1     -1
  1     1    -1     1    -1    -1    -1     1     1     1     -1      1

Here is an alternative way of doing the same thing using a KEY= data
set.

   data key;
     array Had[11];
     input Had1 @@;
     do i = 2 to 11; Had[i] = Had1; end;
     drop i;
     datalines;
   -1 1
   ;

   proc print data=key; run;
   %mktlab(data=design, key=key, int=Had0);

Obs  Had1  Had2  Had3  Had4  Had5  Had6  Had7  Had8  Had9  Had10  Had11

 1    -1    -1    -1    -1    -1    -1    -1    -1    -1     -1     -1
 2     1     1     1     1     1     1     1     1     1      1      1

The Hadamard matrix from this step (not shown) is exactly the same as above.

The KEY= data set contains all of the variables that you want in the
design and a list of their values.  This information will be applied to
the design, by default the one stored in a data set called Randomized,
which is the default OUTR= data set name from MktEx.  The results are
stored in a new data set, Final, with the desired factor names and
levels.

Consider the consumer food product example from Kuhfeld, Tobias, and
Garratt, 1994, "Efficient Experimental Design with Marketing Research
Applications," Journal of Marketing Research, 31, 545-557.  Here is one
possible design.

   data randomized;
     input x1-x8 @@;
     datalines;
   4 2 1 1 1 2 2 2 2 1 1 2 1 3 1 3 3 4 2 2 1 3 2 3 4 3 2 1 3 2 2 3 4 1 2 1
   1 1 1 1 2 4 1 2 1 2 1 1 1 2 1 2 3 3 2 1 2 2 2 2 2 2 2 3 1 4 2 1 1 2 2 2
   3 2 2 1 3 1 2 1 1 4 1 2 2 3 1 2 1 3 2 2 1 3 1 1 3 2 1 2 2 1 2 3 3 4 1 1
   3 1 1 3 4 1 2 2 2 1 2 1 2 3 2 1 2 3 2 2 2 1 2 1 3 3 1 3 4 2 2 2 1 3 1 2
   2 4 2 2 3 1 1 2 3 1 2 2 3 2 1 2 3 3 1 1 2 3 1 1 4 4 2 1 2 2 1 3 1 1 1 1
   3 2 1 2 4 3 1 2 3 3 2 2 1 2 2 1 2 1 1 3 1 3 1 1 1 1 2 3
   ;

Designs created by the MktEx macro always have factor names x1, x2, ...,
and so on, and the levels are consecutive integers beginning with 1 (1,
2 for two-level factors; 1, 2, 3 for three-level factors; and so on).
The MktLab macro provides you with a convenient way to change the names
and levels to more meaningful values.

The data set KEY contains the variable names and levels that you
ultimately want.  The variable Client with 4 levels will be made from
x1, ClientLineExtension with 4 levels will be made from x2, ClientMicro
with 2 levels will be made from x3, and so on.  The N (for not
available) is treated as a special missing value.  The key data set has
four rows because the maximum number of levels is four.  Factors with
fewer than four levels are filled in with ordinary missing values.  The
macro MktLab takes by default the DATA=RANDOMIZED data set from MktEx
and the KEY=KEY data set, and combines the information to create the
OUT=Final data set, which is shown below, sorted by the shelf talker
variable.

   data key;
     missing N;
     input Client ClientLineExtension ClientMicro $ ShelfTalker $
           Regional Private PrivateMicro $ NationalLabel;
     format _numeric_ dollar5.2;
     datalines;
   1.29 1.39 micro Yes 1.99 1.49 micro 1.99
   1.69 1.89 stove No  2.49 2.29 stove 2.39
   2.09 2.39 .     .   N     N   .     N
   N    N    .     .   .     .   .     .
   ;

   %mktlab(key=key);

   proc sort; by shelftalker; run;

   proc print; by shelftalker; run;

Here is the design:

   ----------------------------- ShelfTalker=No ------------------------------

                   Client
                    Line      Client                        Private   National
   Obs   Client   Extension   Micro    Regional   Private    Micro     Label

     1   $1.69      $1.39     micro     $1.99        N       micro       N
     2   $2.09        N       stove     $1.99        N       stove       N
     3   $1.69        N       micro     $1.99      $2.29     micro     $1.99
     4   $1.29      $1.89     micro       N          N       stove     $1.99
     5   $1.69      $1.89     stove     $2.49      $2.29     stove       N
     6   $1.29        N       micro     $2.49        N       micro     $2.39
     7   $1.29      $2.39     stove     $1.99        N       micro     $1.99
     8   $2.09      $1.89     micro     $2.49      $1.49     stove       N
     9     N        $1.39     stove     $2.49      $1.49     stove     $1.99
    10     N        $1.89     stove     $1.99        N       micro     $2.39
    11   $1.69        N       stove       N        $1.49     micro     $2.39
    12   $2.09      $1.39     stove       N        $2.29     micro     $2.39
    13     N        $2.39     micro       N          N       stove     $2.39


   ----------------------------- ShelfTalker=Yes -----------------------------

                   Client
                    Line      Client                        Private   National
   Obs   Client   Extension   Micro    Regional   Private    Micro     Label

    14     N        $1.89     micro     $1.99      $2.29     stove     $2.39
    15     N        $2.39     stove       N        $2.29     stove       N
    16     N        $1.39     stove     $1.99      $1.49     micro     $1.99
    17   $1.29        N       stove     $1.99      $2.29     stove     $2.39
    18   $2.09      $1.89     stove       N        $1.49     stove     $1.99
    19   $2.09        N       micro       N        $1.49     micro       N
    20   $1.69      $2.39     stove     $2.49        N       stove     $2.39
    21   $1.69      $1.39     stove       N          N       micro       N
    22   $2.09      $2.39     micro     $2.49        N       micro     $1.99
    23     N          N       stove     $2.49      $2.29     micro       N
    24   $1.29      $1.39     micro       N        $2.29     micro     $2.39
    25   $1.29      $1.89     stove     $2.49      $1.49     micro       N
    26   $1.29      $2.39     micro     $1.99      $1.49     stove       N

What this macro does is create the OUT= data set by repeatedly reading
and rereading the KEY= data set, one datum at a time, using the
information in the DATA= data set.  In this example, for the first
observation, x1=4 so the fourth value of the first KEY= variable is
read, then x2=2 so the second value of the second KEY= variable is read,
then x3=1 so the first value of the third KEY= variable is read, ...
then x8=2 so the second value of the eighth KEY= variable is read, then
the first observation is output.  This continues for all observations.
This is why the DATA= data set must have integer values beginning with 1.

This example creates the L36, renames the two-level factors two1-two11
and assigns them values -1, 1, and renames the three-level factors
three1-three12 and assigns them values -1, 0, 1.

   %mktex(n=36)

   data key;
     array x[23] two1-two11 three1-three12;
     input two1 three1;
     do i =  2 to 11; x[i] = two1;   end;
     do i = 13 to 23; x[i] = three1; end;
     drop i;
     datalines;
   -1 -1
    1  0
    .  1
   ;

   %mktlab(key=key);

This next step creates a design and blocks it.  This example shows that
it is OK if not all of the variables in the input design are used.  Some
like x4 can be discarded.  Others like Block and Run are just copied
from the input to the output.

   %mktex(n=18, seed=17)
   %mktblock(nblocks=2, factors=x1-x4)

   data key;
      input Brand $ Price Size;
      format price dollar5.2;
      datalines;
   Acme 1.49   6
   Apex 1.79   8
   .    1.99  12
   ;

   %mktlab(data=blocked, key=key)

   proc print; id block run; by block; run;

Here are the results:

              Block    Run    Brand    Price    Size    x4

                1       1     Acme     $1.49     12      2
                        2     Acme     $1.79      6      3
                        3     Acme     $1.79      8      2
                        4     Acme     $1.99     12      1
                        5     Apex     $1.49      8      1
                        6     Apex     $1.49      8      3
                        7     Apex     $1.79     12      1
                        8     Apex     $1.99      6      2
                        9     Apex     $1.99      6      3

                2       1     Acme     $1.49      6      1
                        2     Acme     $1.49      6      2
                        3     Acme     $1.79      8      3
                        4     Acme     $1.99      8      1
                        5     Acme     $1.99     12      3
                        6     Apex     $1.49     12      3
                        7     Apex     $1.79      6      1
                        8     Apex     $1.79     12      2
                        9     Apex     $1.99      8      2

This next example illustrates using the LABELS= option.  This option is
more typically used with VALUES= input, rather than when you construct
the KEY= data set yourself, but it can be used either way.  This example
is from the "Vacation Example, Big Designs" in "Multinomial Logit,
Discrete Choice Modeling."

   %mktex(3 ** 15, n=36, seed=17, maxtime=0)

   %mkteval;

   %mktblock(data=randomized, nblocks=2, factors=x1-x15)

   %macro lab;
      label X1  = 'Hawaii, Accommodations'
            X2  = 'Alaska, Accommodations'
            X3  = 'Mexico, Accommodations'
            X4  = 'California, Accommodations'
            X5  = 'Maine, Accommodations'
            X6  = 'Hawaii, Scenery'
            X7  = 'Alaska, Scenery'
            X8  = 'Mexico, Scenery'
            X9  = 'California, Scenery'
            X10 = 'Maine, Scenery'
            X11 = 'Hawaii, Price'
            X12 = 'Alaska, Price'
            X13 = 'Mexico, Price'
            X14 = 'California, Price'
            X15 = 'Maine, Price';
      format x11-x15 dollar5.;
   %mend;

   data key;
      length x1-x5 $ 16 x6-x10 $ 8 x11-x15 8;
      input x1 & $ x6 $ x11;
      x2  = x1;    x3 = x1;    x4 = x1;    x5 = x1;
      x7  = x6;    x8 = x6;    x9 = x6;   x10 = x6;
      x12 = x11;  x13 = x11;  x14 = x11;  x15 = x11;
      datalines;
   Cabin            Mountains   999
   Bed & Breakfast  Lake       1249
   Hotel            Beach      1499
   ;

   %mktlab(data=blocked, key=key, labels=lab)

   proc contents p; ods select position; run;

Here is the variable name, label, and format information.

                     -----Variables Ordered by Position-----

     #   Variable   Type   Len   Pos   Format     Label
    ------------------------------------------------------------------------
     1   x1         Char    16    56              Hawaii, Accommodations
     2   x2         Char    16    72              Alaska, Accommodations
     3   x3         Char    16    88              Mexico, Accommodations
     4   x4         Char    16   104              California, Accommodations
     5   x5         Char    16   120              Maine, Accommodations
     6   x6         Char     8   136              Hawaii, Scenery
     7   x7         Char     8   144              Alaska, Scenery
     8   x8         Char     8   152              Mexico, Scenery
     9   x9         Char     8   160              California, Scenery
    10   x10        Char     8   168              Maine, Scenery
    11   x11        Num      8     0   DOLLAR5.   Hawaii, Price
    12   x12        Num      8     8   DOLLAR5.   Alaska, Price
    13   x13        Num      8    16   DOLLAR5.   Mexico, Price
    14   x14        Num      8    24   DOLLAR5.   California, Price
    15   x15        Num      8    32   DOLLAR5.   Maine, Price
    16   Block      Num      8    40
    17   Run        Num      8    48

------------------------------------------------------------------------

 DISCLAIMER:

       THIS INFORMATION IS PROVIDED BY SAS INSTITUTE INC. AS A SERVICE
 TO ITS USERS.  IT IS PROVIDED "AS IS".  THERE ARE NO WARRANTIES,
 EXPRESSED OR IMPLIED, AS TO MERCHANTABILITY OR FITNESS FOR A
 PARTICULAR PURPOSE REGARDING THE ACCURACY OF THE MATERIALS OR CODE
 CONTAINED HEREIN.

----------------------------------------------------------------------*/

%*===================== Macro Argument List Start =====================;

%macro mktlab(    /*--------------------------------------------------*/
                  /*                                                  */
help,             /* nonblank, list options                           */
                  /*                                                  */
data=Randomized,  /* specifies the input data set with the            */
                  /* experimental design, usually created by the      */
                  /* MktEx macro.                                     */
                  /*                                                  */
key=,             /* specifies the input data set with the key to     */
                  /* recoding the design.  When VALUES= or DOLIST= is */
                  /* specified, this data set is made for you.        */
                  /* By default, when KEY=, VALUES=, and DOLIST=      */
                  /* are all not specified, the default value list    */
                  /* comes from DOLIST=1 to 100.                      */
                  /*                                                  */
out=Final,        /* specifies the output data set with the final,    */
                  /* recoded design.                                  */
                  /*                                                  */
int=,             /* specifies the name of an intercept variable      */
                  /* (column of ones), if you want an intercept added */
                  /* to the OUT= data set.  You can also specify a    */
                  /* variable list instead of a variable name if you  */
                  /* would like to make a list of variables with      */
                  /* values all one.  This can be useful for example, */
                  /* for creating flags for generic alternatives when */
                  /* the design is a candidate set for making a       */
                  /* choice design with the ChoicEff macro.           */
                  /*                                                  */
values=,          /* specifies the new values for all of the          */
                  /* variables.  If all variables will have the same  */
                  /* value, it is easier to specify VALUES= or        */
                  /* DOLIST= than KEY=.  When you specify VALUES=,    */
                  /* the KEY= data set is created for you.  Specify a */
                  /* list of levels separated by blanks.  If your     */
                  /* levels contain blanks, separate them with two    */
                  /* blanks.  With asymmetric designs (not all        */
                  /* factors have the same levels) specify the levels */
                  /* for the largest number of levels.  For example,  */
                  /* with two-level and three-level factors and       */
                  /* VALUES=a b c, the two-level factors will be      */
                  /* assigned levels 'a' and 'b', and the three-level */
                  /* factors will be assigned levels 'a', 'b', and    */
                  /* 'c'.  Do not specify both VALUES= and DOLIST=.   */
                  /* By default, when KEY=, VALUES=, and DOLIST=      */
                  /* are all not specified, the default value list    */
                  /* comes from DOLIST=1 to 100.                      */
                  /*                                                  */
dolist=,          /* specifies the new values, using a do-list syntax */
                  /* (n TO m <BY p>), for example: 1 to 10 or 0 to 9. */
                  /* With asymmetric designs (not all factors have    */
                  /* the same levels), specify the levels for the     */
                  /* largest number of levels.  For example, with     */
                  /* two-level and three-level factors and            */
                  /* DOLIST=0 to 2, the two-level factors will be     */
                  /* assigned levels 0 and 1, and the three-level     */
                  /* factors will be assigned levels 0, 1, and 2.     */
                  /* Do not specify both VALUES= and DOLIST=.         */
                  /* By default, when KEY=, VALUES=, and DOLIST=      */
                  /* are all not specified, the default value list    */
                  /* comes from DOLIST=1 to 100.                      */
                  /*                                                  */
nfill=.,          /* specifies the fill value in the KEY= data set    */
                  /* for numeric variables.  For example when the     */
                  /* maximum number of levels is three, the last      */
                  /* value in the KEY= data set for numeric two-level */
                  /* factors should have a value of NFILL=, which by  */
                  /* default is ordinary missing.  If the macro tries */
                  /* to access one of these values, it prints a       */
                  /* warning.  If you would like ordinary missing (.) */
                  /* to be a legitimate level, specify a different    */
                  /* NFILL= value and use it for the extra places in  */
                  /* the KEY= data set.                               */
                  /*                                                  */
cfill=' ',        /* specifies the fill value in the KEY= data set    */
                  /* for character variables.  See NFILL= for more    */
                  /* information on fill values.                      */
                  /*                                                  */
vars=,            /* specifies a list of variable names when VALUES=  */
                  /* or DOLIST= is specified.  If VARS= is not        */
                  /* specified with VALUES=, then PREFIX= is used.    */
                  /*                                                  */
prefix=x,         /* specifies a prefix for naming variables when     */
                  /* VALUES= is specified.  By default the variables  */
                  /* are x1, x2, ....  This option is ignored when    */
                  /* VARS= is specified.                              */
                  /*                                                  */
labels=,          /* specifies the name of a macro that provides      */
                  /* labels, formats, or other additional information */
                  /* to the key data set.  For a simple format        */
                  /* specification, it is easier to use STATEMENTS=.  */
                  /* For more involved specifications, use LABELS=.   */
                  /* Note that you specify just the macro name, no    */
                  /* percents on the LABELS= option.  Example:        */
                  /*                                                  */
                  /*    %mktex(3 ** 4, n=18, seed=205)                */
                  /*                                                  */
                  /*    %macro labs;                                  */
                  /*       label x1 = 'Sploosh' x2 = 'Plumbob'        */
                  /*             x3 = 'Platter' x4 = 'Moosey';        */
                  /*       format x1-x4 dollar5.2;                    */
                  /*       %mend;                                     */
                  /*                                                  */
                  /*    %mktlab(values=1.49 1.99 2.49, labels=labs)   */
                  /*                                                  */
                  /*    proc print label; run;                        */
                  /*                                                  */
                  /* Obs    Sploosh    Plumbob    Platter    Moosey   */
                  /*                                                  */
                  /*   1     $1.49      $1.49      $1.99     $2.49    */
                  /*   2     $2.49      $1.99      $1.49     $2.49    */
                  /*   3     $1.49      $1.99      $1.49     $1.99    */
                  /*   4     $1.49      $1.99      $1.99     $1.49    */
                  /*   .                                              */
                  /*   .                                              */
                  /*   .                                              */
                  /*                                                  */
statements=,      /* is an alternative to labels= that you can use    */
stmts=,           /* to add extra statements to the KEY= data set.    */
                  /* For a simple format specification, it is For a   */
                  /* simple format specification, it is easier to use */
                  /* STATEMENTS=.  For more involved specifications,  */
                  /* use LABELS=.  Example:                           */
                  /*                                                  */
                  /*    %mktex(3 ** 4, n=18, seed=205)                */
                  /*                                                  */
                  /*    %mktlab(values=1.49 1.99 2.49,                */
                  /*            vars=Sploosh Plumbob Platter Moosey,  */
                  /*            statements=format Sploosh Plumbob     */
                  /*                       Platter Moosey dollar5.2)  */
                  /*                                                  */
                  /*    proc print; run;                              */
                  /*                                                  */
                  /* Obs    Sploosh    Plumbob    Platter    Moosey   */
                  /*                                                  */
                  /*   1     $1.49      $1.49      $1.99     $2.49    */
                  /*   2     $2.49      $1.99      $1.49     $2.49    */
                  /*   3     $1.49      $1.99      $1.49     $1.99    */
                  /*   4     $1.49      $1.99      $1.99     $1.49    */
                  /*   .                                              */
                  /*   .                                              */
                  /*   .                                              */
                  /*                                                  */
options=,         /* noprint - do not print the variable mapping.     */
                  /*                                                  */
);                /*--------------------------------------------------*/
                  /* This macro specifies OPTIONS NONOTES throughout  */
                  /* most of its execution.  If you want to see all   */
                  /* the notes, submit the statement                  */
                  /*    %let mktopts = notes;                         */
                  /* before running the macro.                        */
                  /*--------------------------------------------------*/

;*=========================== Macro Start =============================;

*------initial macro stuff------;
%global _MktVer;
%local  m i k time abort savenote setnote savedsnf nvals anychar saveopts
        maxlen maxplen localver dowarn timeopt print;

%let time     = %sysfunc(datetime());
%let timeopt  = %scan(STIMER STATS, 1 + (&sysscp = OS));
%let savenote = %sysfunc(getoption(notes));
%let saveopts = &savenote missing="%sysfunc(getoption(missing))"
                %sysfunc(getoption(generic)) %sysfunc(getoption(&timeopt))
                %sysfunc(getoption(dsnferr)) %sysfunc(getoption(fmterr))
                %sysfunc(getoption(serror));
%let setnote  = nonotes;
%let abort    = 0;
%let maxlen   = 32700;
%let localver = 25Jul2010;
%let _mktver  = &localver;

options noserror;
%if %nrquote(&&mktopts) ne %nrstr(&)mktopts %then %do;
   %let k = %upcase(&mktopts);
   %if %index(&k, CHECKVERSION) %then %do; options serror; %goto mendit; %end;
   %if %index(&k, NOTES)   %then %let setnote = notes;
   %if %index(&k, VERSION) %then %do;
      %put MktLab macro version &localver.;
      %put MktLab macro compatibility fix 13Jul2009;
      %end;
   %end;
options serror;

options &setnote generic missing='.';

%if %nrbquote(&help) ne %then %do; %mkthelp(mktlab) %goto endit; %end;

*------mention data sets, if they are invalid, quit------;
filename __dummy dummy;
proc printto log=__dummy; run;
%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

%let savedsnf = %sysfunc(getoption(dsnferr)) %sysfunc(getoption(fmterr));
options nodsnferr nofmterr;
%if %nrbquote(&data) ne %then %do;
   data _null_; set &data; stop; run;
   %if &syserr > 4 %then %let abort = 1;
   %end;
%if %nrbquote(&out) ne %then %do;
   data _null_; set &out; stop; run;
   %if &syserr > 4 %then %let abort = 1;
   %end;
%if %nrbquote(&key) ne %then %do;
   data _null_; set &key; stop; run;
   %if &syserr > 4 %then %let abort = 1;
   %end;
options &savedsnf;

proc printto log=log; run;
%if &syserr > 4 %then %let abort = 2;

%if &abort eq 1 %then %do;
   %put ERROR: One or more data set names are invalid:;
   %put %str(       )outall=&data;
   %put %str(       )out=&out;
   %put %str(       )outr=&key;
   %end;

%if &abort %then %goto endit;

*------make sure we have data sets specified------;
data _null_;
   %if %nrbquote(&data) ne %then %do; set &data; %end;

   if symget('out') eq ' ' or symget('data') eq ' ' then do;
      put "ERROR: A data set name must be specified for DATA= and OUT=.";
      abort = 1;
      end;

   if _error_ or abort then call symput('abort', '1');
   stop;
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------allow for numeric design levels not equal to 1, 2, 3 ...------;
%if %sysevalf(&sysver ge 9.2) %then %do;
   proc rank data=&data out=__tmpldata ties=dense; var x:; run;
   %end;

%else %do;
   data __tmpldata; set &data; if _error_ then call symput('abort', '1'); run;
   %end;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------count vars, set default lists------;
data _null_;

   *------count DATA= vars------;
   if 0 then set __tmpldata;
   array __x[*] x:;
   if _n_ = 1 then call symput('m', compress(put(dim(__x), 12.)));

   *------default lists------;
   if symget('key')    eq ' ' and symget('dolist') eq ' ' and
      symget('values') eq ' ' then call symput('dolist', '1 to 100');
   if symget('key')    eq ' ' then call symput('key', '__tmplabkey');

   *------check for invalid do list------;
   if indexc(symget('dolist'), "'" || '"') then do;
      put 'ERROR: DOLIST= may not contain quoted strings.';
      call symput('abort', '1');
      end;

   if _error_ then call symput('abort', '1');
   stop;
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------basic parameter checks------;
data _null_;

   length &prefix.1 8 to $ 8;

   *------initialize stuff------;
   abort = 0;
   length w w2 $ 200 list $ &maxlen;
   %if %nrbquote(&int) ne %then %do; length &int 8; %end;
   nvals = 1;
   anychar = 0;

   *------options------;
   list = lowcase(symget('options'));
   i = index(list, 'noprint'); if i then substr(list, i, 7) = ' ';
   call symput('print', put(i eq 0, 1.));
   if list ne ' ' then do;
      list = compbl(list);
      put 'ERROR: OPTIONS=' list 'is invalid.';
      abort = 1;
      end;

   list = symget('values');

   if list eq ' ' then do;
      %if %nrbquote(&dolist) ne %then %do;
         do i = &dolist;
            list = trim(list) || ' ' || put(i, best12. -L);
            end;
         %end;
      end;

   else if symget('dolist') ne ' ' then do;
      put 'ERROR: Do not specify both VALUES= and DOLIST=.';
      abort = 1;
      end;

   *-------set array name with VARS=------;
   if symget('vars') ne ' ' then do;
      call symput('prefix', '__x');
      if list eq ' ' then do;
         put 'ERROR: You must specify VALUES= or DOLIST= '
             'when you specify VARS=.';
         abort = 1;
         end;
      end;

   *------parse VALUES= list------;
   maxplen = 8;
   call symput('dowarn', '0');
   if list ne ' ' then do;
      list = left(list);
      i = input(scan(list, 1, ' '), ?? 23.);
      j = input(scan(list, 3, ' '), ?? 23.);
      to = upcase(scan(list, 2, ' '));
      if n(i,j) eq 2 and i lt j and to eq 'TO' then call symput('dowarn', '1');
      i = index(trim(list), '  ');
      if i then do;
         do until(i = 0);
            substr(list, i, 2)  = '0000'x;
            i = index(trim(list), '  ');
            end;
         end;
      else list = translate(list, '00'x, ' ');
      do nvals = 1 to &maxlen until(w = ' ');
         w = left(scan(list, nvals, '00'x));
         if w ne ' ' then do;
            maxplen = max(maxplen, length(w));
            w2      = upcase(w);
            char    = nmiss(input(w, ?? 24.));
            if char and length(w2) le 2 then do;
               do m = '. ', '.A', '.B', '.C', '.D', '.E', '.F',
                      '.G', '.H', '.U', '.J', '.K', '.L', '.M',
                      '.N', '.O', '.P', '.Q', '.R', '.S', '.T',
                      '.U', '.V', '.W', '.X', '.Y', '.Z', '._';
                  if w2 eq m then char = 0;
                  end;
               end;
            if length(w) gt 24 or char then anychar = 1;
            call symput('__val' || compress(put(nvals, 5.)), trim(w));
            end;
         end;
      end;

   call symput('nvals',   compress(put(nvals - 1, 5.)));
   call symput('anychar', put(anychar, 1.));
   call symput('maxplen', put(maxplen, 4.));

   *------we need a prefix------;
   if symget('prefix') = ' ' then do;
      put "ERROR: PREFIX=' ' is invalid.";
      abort = 1;
      end;

   if abort or _error_ then call symput('abort', '1');
   stop;
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------if VALUES= was specified, make KEY= data set------;
%if &nvals %then %do;

   data &key;
      %if %nrbquote(&vars) eq and &anychar %then %do;
         length &prefix.1-&prefix&m $ &maxplen;
         %end;
      array &prefix[%if %nrbquote(&vars) eq %then &m;%else *;]
            %if &anychar %then $;
            %if %nrbquote(&vars) ne %then &vars;;
      %do i = 1 %to &nvals;
         &prefix[1] = %if &anychar %then "&&__val&i"; %else &&__val&i;;
         do __i = 2 to dim(&prefix); &prefix[__i] = &prefix[1]; end;
         output;
         %end;

      &statements; &stmts;
      %if %nrbquote(&labels) ne %then %do; %&labels; %end;
      drop __i;
      if _error_ then call symput('abort', '1');
      run;

   %if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

   %end;

*------get the names and types of the KEY= data set vars------;
proc contents data=&key out=__tmplabnam(keep=name type varnum) noprint; run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

*------get the var names in order------;
proc sort data=__tmplabnam; by varnum; run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

data _null_;

   *------rudimentary checks on the DATA= data set------;
   if 0 then set __tmpldata;
   if _n_ = 1 then do;
      array __x[*] x:;
      abort = 0;
      if dim(__x) lt m then do;
         put 'ERROR: The DATA= data set must have at least as '
             'many X variables as variables in the KEY= data set.';
         abort = 1;
         goto endit;
         end;
      if &print then put +3 'Variable Mapping:';
      %if &print and %nrbquote(&int) ne %then %do;
         put +6 '1' +(2 + (m > 9) + (m > 99)) ": &int";
         %end;
      end;

   *------put the var names and types in macro variables------;
   set __tmplabnam nobs=m;
   if _n_ = 1 then call symput('m', compress(put(m, 12.)));
   call symput('__var' || compress(put(_n_, 12.)), trim(name));
   call symput('__num' || compress(put(_n_, 12.)), put(type eq 1, 1.));
   if &print then
      put +6 'x' _n_ +((m > 9) + (m > 99) - (_n_ > 9) - (_n_ > 99)) ': ' name;

   endit: if abort or _error_ then call symput('abort', '1');
   run;

%if &syserr > 4 %then %let abort = 1;
%if &abort %then %do;
   proc contents p data=&key; ods select position; run;
   %goto endit;
   %end;

options &savenote no&timeopt;

*------read the data= data set and change the names and values------;
data &out;

   %if %nrbquote(&int) ne %then %do; retain &int 1; %end;

   *-------get the right var names first------;
   if 0 then set &key nobs=__n;

   *------now read a row of the design, avoid name conflicts------;
   set __tmpldata(rename=(%do i = 1 %to &m; x&i = __&i %end;));

   %do i = 1 %to &m;

      *------for each var, we need to get the ith level------;
      __i = __&i;

      *------make sure we have a valid integer level number------;
      if nmiss(__i) or __i < 0 or __i > __n or __i ne round(__i) then do;
         put "ERROR: Variable x&i's value " __i
             'must be an integer between 1 and ' __n +(-1) '.';
         if &dowarn then put "WARNING: Did you mean to specify DOLIST=&values "
            "instead of VALUES=&values.?";
         call symput('abort', '1');
         stop;
         end;

      *------now grab the new value from the key data set------;
      set &key(keep=&&__var&i) point=__i;

      *------make sure we got a nonmissing value, special missing OK------;
      if %if &&__num&i %then &&__var&i eq &nfill; %else &&__var&i eq &cfill; then
         put "WARNING: Variable &&__Var&i has a fill value "
             'in observation ' _n_ +(-1) '.';
      %end;

   &statements;
   %if %nrbquote(&labels) ne %then %do; %&labels; %end;

   drop __:;

   if _error_ then call symput('abort', '1');
   run;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

options nonotes;
proc datasets nolist;
   delete __tmplabkey __tmplabnam __tmpldata;
   run; quit;

%if &syserr > 4 %then %let abort = 1; %if &abort %then %goto endit;

%endit: options &saveopts;
quit;

%if &abort %then %do;
   %if &syserr = 1016 or &syserr = 116  %then %put ERROR: Insufficient memory.;
   %if &syserr = 2000 or &syserr = 3000 %then
      %put ERROR: Syntax error.  Check your macro arguments for validity.;
   %put ERROR: The MKTLAB macro ended abnormally.;
   %end;

%if %sysfunc(getoption(&timeopt)) eq &timeopt and
   %sysfunc(getoption(stimer)) eq STIMER %then %do;
   %let time = %sysfunc(round(%sysevalf(%sysfunc(datetime()) - &time), 0.01));
   %if %sysevalf(&time >= 60) %then %do;
      %let time = %sysfunc(round(%sysevalf(&time / 60), 0.01)) minutes.;
      %end;
   %else %let time = &time seconds.;
   %put NOTE: The MKTLAB macro used &time;
   %end;

%mendit: %mend;
